/* Generated by Cython 0.27.1 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [], 
        "name": "pyrogg", 
        "sources": [
            "/home/stefan/source/Python/pyogg/pyrogg/src/pyrogg.pyx"
        ]
    }, 
    "module_name": "pyrogg"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_1"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #ifdef __cplusplus
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough) || (defined(__GNUC__) && defined(__attribute__))
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyrogg
#define __PYX_HAVE_API__pyrogg
#include "ogg/ogg.h"
#include <string.h>
#include <stdio.h>
#include "vorbis/codec.h"
#include "vorbis/vorbisfile.h"
#include "vorbis/vorbisenc.h"
#include <errno.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/pyrogg.pyx",
  "stringsource",
  "type.pxd",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/*--- Type declarations ---*/
struct __pyx_obj_6pyrogg_VorbisFile;
struct __pyx_obj_6pyrogg__VorbisRecoder;
struct __pyx_obj_6pyrogg_VorbisFileRecoder;
struct __pyx_obj_6pyrogg_FilelikeReader;
struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder;
struct __pyx_t_6pyrogg_communication;

/* "pyrogg.pyx":81
 * 
 * 
 * ctypedef int (*page_writer_function)(void*, ogg.ogg_page*) nogil             # <<<<<<<<<<<<<<
 * 
 * cdef struct communication:   # thread communication
 */
typedef int (*__pyx_t_6pyrogg_page_writer_function)(void *, ogg_page *);

/* "pyrogg.pyx":83
 * ctypedef int (*page_writer_function)(void*, ogg.ogg_page*) nogil
 * 
 * cdef struct communication:   # thread communication             # <<<<<<<<<<<<<<
 *     int done
 *     int buffer_ready
 */
struct __pyx_t_6pyrogg_communication {
  int done;
  int buffer_ready;
  int next_buffer_ready;
  long read_status[2];
};

/* "pyrogg.pyx":55
 * 
 * 
 * cdef class VorbisFile:             # <<<<<<<<<<<<<<
 *     cdef vorbis.OggVorbis_File _vorbisfile
 *     cdef object filename
 */
struct __pyx_obj_6pyrogg_VorbisFile {
  PyObject_HEAD
  OggVorbis_File _vorbisfile;
  PyObject *filename;
};


/* "pyrogg.pyx":90
 * 
 * 
 * cdef class _VorbisRecoder:             # <<<<<<<<<<<<<<
 *     cdef page_writer_function _write
 * 
 */
struct __pyx_obj_6pyrogg__VorbisRecoder {
  PyObject_HEAD
  struct __pyx_vtabstruct_6pyrogg__VorbisRecoder *__pyx_vtab;
  __pyx_t_6pyrogg_page_writer_function _write;
};


/* "pyrogg.pyx":289
 * 
 * 
 * cdef class VorbisFileRecoder(_VorbisRecoder):             # <<<<<<<<<<<<<<
 *     cdef object _input_filename
 * 
 */
struct __pyx_obj_6pyrogg_VorbisFileRecoder {
  struct __pyx_obj_6pyrogg__VorbisRecoder __pyx_base;
  PyObject *_input_filename;
};


/* "pyrogg.pyx":340
 * 
 * @cython.final
 * cdef class FilelikeReader:             # <<<<<<<<<<<<<<
 *     cdef object read
 *     cdef object tell
 */
struct __pyx_obj_6pyrogg_FilelikeReader {
  PyObject_HEAD
  struct __pyx_vtabstruct_6pyrogg_FilelikeReader *__pyx_vtab;
  PyObject *read;
  PyObject *tell;
  PyObject *seek;
  PyObject *close;
  PyObject *exception;
};


/* "pyrogg.pyx":364
 * 
 * 
 * cdef class VorbisFilelikeRecoder(_VorbisRecoder):             # <<<<<<<<<<<<<<
 *     cdef object f
 * 
 */
struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder {
  struct __pyx_obj_6pyrogg__VorbisRecoder __pyx_base;
  PyObject *f;
};



/* "pyrogg.pyx":90
 * 
 * 
 * cdef class _VorbisRecoder:             # <<<<<<<<<<<<<<
 *     cdef page_writer_function _write
 * 
 */

struct __pyx_vtabstruct_6pyrogg__VorbisRecoder {
  PyObject *(*_recode)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, OggVorbis_File *, int);
  PyObject *(*_recodeParallel)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, char *, OggVorbis_File *, vorbis_dsp_state, vorbis_block, ogg_stream_state);
  int (*_writeVorbisHeader)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_comment *);
  int (*_encodeVorbisBlocks)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_block *);
  int (*_writeOggPackets)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *);
};
static struct __pyx_vtabstruct_6pyrogg__VorbisRecoder *__pyx_vtabptr_6pyrogg__VorbisRecoder;
static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recode(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, OggVorbis_File *, int);
static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recodeParallel(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, char *, OggVorbis_File *, vorbis_dsp_state, vorbis_block, ogg_stream_state);
static int __pyx_f_6pyrogg_14_VorbisRecoder__writeVorbisHeader(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_comment *);
static int __pyx_f_6pyrogg_14_VorbisRecoder__encodeVorbisBlocks(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_block *);
static int __pyx_f_6pyrogg_14_VorbisRecoder__writeOggPackets(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *);


/* "pyrogg.pyx":289
 * 
 * 
 * cdef class VorbisFileRecoder(_VorbisRecoder):             # <<<<<<<<<<<<<<
 *     cdef object _input_filename
 * 
 */

struct __pyx_vtabstruct_6pyrogg_VorbisFileRecoder {
  struct __pyx_vtabstruct_6pyrogg__VorbisRecoder __pyx_base;
};
static struct __pyx_vtabstruct_6pyrogg_VorbisFileRecoder *__pyx_vtabptr_6pyrogg_VorbisFileRecoder;


/* "pyrogg.pyx":340
 * 
 * @cython.final
 * cdef class FilelikeReader:             # <<<<<<<<<<<<<<
 *     cdef object read
 *     cdef object tell
 */

struct __pyx_vtabstruct_6pyrogg_FilelikeReader {
  void (*_storeException)(struct __pyx_obj_6pyrogg_FilelikeReader *, PyObject *);
};
static struct __pyx_vtabstruct_6pyrogg_FilelikeReader *__pyx_vtabptr_6pyrogg_FilelikeReader;
static void __pyx_f_6pyrogg_14FilelikeReader__storeException(struct __pyx_obj_6pyrogg_FilelikeReader *, PyObject *);


/* "pyrogg.pyx":364
 * 
 * 
 * cdef class VorbisFilelikeRecoder(_VorbisRecoder):             # <<<<<<<<<<<<<<
 *     cdef object f
 * 
 */

struct __pyx_vtabstruct_6pyrogg_VorbisFilelikeRecoder {
  struct __pyx_vtabstruct_6pyrogg__VorbisRecoder __pyx_base;
};
static struct __pyx_vtabstruct_6pyrogg_VorbisFilelikeRecoder *__pyx_vtabptr_6pyrogg_VorbisFilelikeRecoder;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   ((obj)->ob_type->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
static int __Pyx_CLineForTraceback(int c_line);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ogg_int64_t(ogg_int64_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recode(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, OggVorbis_File *__pyx_v_vorbisfile, int __pyx_v_target_quality); /* proto*/
static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recodeParallel(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, char *__pyx_v_decbuffer, OggVorbis_File *__pyx_v_vorbisfile, vorbis_dsp_state __pyx_v_dsp_state, vorbis_block __pyx_v_vorbis_block, ogg_stream_state __pyx_v_stream_state); /* proto*/
static int __pyx_f_6pyrogg_14_VorbisRecoder__writeVorbisHeader(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state, vorbis_comment *__pyx_v_comment); /* proto*/
static int __pyx_f_6pyrogg_14_VorbisRecoder__encodeVorbisBlocks(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state, vorbis_block *__pyx_v_vorbis_block); /* proto*/
static int __pyx_f_6pyrogg_14_VorbisRecoder__writeOggPackets(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state); /* proto*/
static void __pyx_f_6pyrogg_14FilelikeReader__storeException(struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, PyObject *__pyx_v_exception); /* proto*/

/* Module declarations from 'ogg' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'vorbis' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython' */

/* Module declarations from 'cython' */

/* Module declarations from 'libc' */

/* Module declarations from 'libc.errno' */

/* Module declarations from 'pyrogg' */
static PyTypeObject *__pyx_ptype_6pyrogg_VorbisFile = 0;
static PyTypeObject *__pyx_ptype_6pyrogg__VorbisRecoder = 0;
static PyTypeObject *__pyx_ptype_6pyrogg_VorbisFileRecoder = 0;
static PyTypeObject *__pyx_ptype_6pyrogg_FilelikeReader = 0;
static PyTypeObject *__pyx_ptype_6pyrogg_VorbisFilelikeRecoder = 0;
static PyObject *__pyx_v_6pyrogg_time = 0;
static PyObject *__pyx_v_6pyrogg_sys = 0;
static ov_callbacks __pyx_v_6pyrogg_filelike_callbacks;
static int __pyx_f_6pyrogg__checkVorbisResult(int); /*proto*/
static PyObject *__pyx_f_6pyrogg__encodeFilename(PyObject *); /*proto*/
static int __pyx_f_6pyrogg__writeToFile(void *, ogg_page *); /*proto*/
static int __pyx_f_6pyrogg__writeToFilelike(void *, ogg_page *); /*proto*/
static size_t __pyx_f_6pyrogg__readFilelike(void *, size_t, size_t, void *); /*proto*/
static int __pyx_f_6pyrogg__seekFilelike(void *, ogg_int64_t, int); /*proto*/
static long __pyx_f_6pyrogg__tellFilelike(void *); /*proto*/
static int __pyx_f_6pyrogg__closeFilelike(void *); /*proto*/
static void __pyx_f_6pyrogg__splitStereo(char *, float *, float *, int); /*proto*/
static int __pyx_f_6pyrogg__initVorbisEncoder(ogg_stream_state *, vorbis_dsp_state *, vorbis_info *, int); /*proto*/
#define __Pyx_MODULE_NAME "pyrogg"
int __pyx_module_is_main_pyrogg = 0;

/* Implementation of 'pyrogg' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_IOError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_f[] = "f";
static const char __pyx_k_0_2[] = "0.2";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_read[] = "read";
static const char __pyx_k_seek[] = "seek";
static const char __pyx_k_tell[] = "tell";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_pyrogg[] = "pyrogg";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_quality[] = "quality";
static const char __pyx_k_version[] = "__version__";
static const char __pyx_k_exc_info[] = "exc_info";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_iso8859_1[] = "iso8859-1";
static const char __pyx_k_lost_sync[] = "lost sync";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_input_filename[] = "input_filename";
static const char __pyx_k_VorbisException[] = "VorbisException";
static const char __pyx_k_output_filename[] = "output_filename";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_Opening_file_r_failed[] = "Opening file %r failed";
static const char __pyx_k_VorbisDecodeException[] = "VorbisDecodeException";
static const char __pyx_k_getfilesystemencoding[] = "getfilesystemencoding";
static const char __pyx_k_r_is_not_a_Vorbis_file[] = "%r is not a Vorbis file";
static const char __pyx_k_expected_file_path_got_s[] = "expected file path, got %s";
static const char __pyx_k_Error_reading_from_file_r[] = "Error reading from file %r";
static const char __pyx_k_file_is_not_a_Vorbis_file[] = "file is not a Vorbis file";
static const char __pyx_k_Error_opening_output_file_r[] = "Error opening output file %r";
static const char __pyx_k_Returned_string_is_too_long[] = "Returned string is too long";
static const char __pyx_k_Error_reading_from_file_like[] = "Error reading from file-like";
static const char __pyx_k_self__write_cannot_be_converted[] = "self._write cannot be converted to a Python object for pickling";
static const char __pyx_k_Invalid_return_type_from_read_by[] = "Invalid return type from read(), bytes required, got %s";
static const char __pyx_k_Seekable_file_like_object_requir[] = "Seekable file-like object required, got %s";
static const char __pyx_k_Unknown_error_d_occurred_while_p[] = "Unknown error %d occurred while preparing recoder";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_writable_file_like_object_requir[] = "writable file-like object required, got %s";
static PyObject *__pyx_kp_s_0_2;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_kp_s_Error_opening_output_file_r;
static PyObject *__pyx_kp_s_Error_reading_from_file_like;
static PyObject *__pyx_kp_s_Error_reading_from_file_r;
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_kp_s_Invalid_return_type_from_read_by;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_Opening_file_r_failed;
static PyObject *__pyx_kp_s_Returned_string_is_too_long;
static PyObject *__pyx_kp_s_Seekable_file_like_object_requir;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unknown_error_d_occurred_while_p;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_VorbisDecodeException;
static PyObject *__pyx_n_s_VorbisException;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_exc_info;
static PyObject *__pyx_kp_s_expected_file_path_got_s;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_kp_s_file_is_not_a_Vorbis_file;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_n_s_getfilesystemencoding;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_input_filename;
static PyObject *__pyx_kp_s_iso8859_1;
static PyObject *__pyx_kp_s_lost_sync;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_output_filename;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pyrogg;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_quality;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_kp_s_r_is_not_a_Vorbis_file;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_read;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_seek;
static PyObject *__pyx_kp_s_self__write_cannot_be_converted;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_tell;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_version;
static PyObject *__pyx_kp_s_writable_file_like_object_requir;
static PyObject *__pyx_n_s_write;
static int __pyx_pf_6pyrogg_10VorbisFile___cinit__(struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self, PyObject *__pyx_v_filename); /* proto */
static void __pyx_pf_6pyrogg_10VorbisFile_2__dealloc__(struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_10VorbisFile_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_10VorbisFile_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6pyrogg_14_VorbisRecoder___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_14_VorbisRecoder_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6pyrogg_17VorbisFileRecoder___cinit__(struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, PyObject *__pyx_v_input_filename); /* proto */
static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_2recode(struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, PyObject *__pyx_v_output_filename, int __pyx_v_quality); /* proto */
static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6pyrogg_14FilelikeReader___cinit__(struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, PyObject *__pyx_v_f); /* proto */
static PyObject *__pyx_pf_6pyrogg_14FilelikeReader_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_14FilelikeReader_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6pyrogg_21VorbisFilelikeRecoder___cinit__(struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, PyObject *__pyx_v_f); /* proto */
static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_2recode(struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, PyObject *__pyx_v_f, int __pyx_v_quality); /* proto */
static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6pyrogg_VorbisFile(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6pyrogg__VorbisRecoder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6pyrogg_VorbisFileRecoder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6pyrogg_FilelikeReader(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6pyrogg_VorbisFilelikeRecoder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;

/* "pyrogg.pyx":38
 * 
 * 
 * cdef int _checkVorbisResult(int result_code) except -1:             # <<<<<<<<<<<<<<
 *     if result_code < 0:
 *         raise VorbisException()
 */

static int __pyx_f_6pyrogg__checkVorbisResult(int __pyx_v_result_code) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_checkVorbisResult", 0);

  /* "pyrogg.pyx":39
 * 
 * cdef int _checkVorbisResult(int result_code) except -1:
 *     if result_code < 0:             # <<<<<<<<<<<<<<
 *         raise VorbisException()
 *     return 0
 */
  __pyx_t_1 = ((__pyx_v_result_code < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":40
 * cdef int _checkVorbisResult(int result_code) except -1:
 *     if result_code < 0:
 *         raise VorbisException()             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 40, __pyx_L1_error)

    /* "pyrogg.pyx":39
 * 
 * cdef int _checkVorbisResult(int result_code) except -1:
 *     if result_code < 0:             # <<<<<<<<<<<<<<
 *         raise VorbisException()
 *     return 0
 */
  }

  /* "pyrogg.pyx":41
 *     if result_code < 0:
 *         raise VorbisException()
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef bytes _encodeFilename(filename):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyrogg.pyx":38
 * 
 * 
 * cdef int _checkVorbisResult(int result_code) except -1:             # <<<<<<<<<<<<<<
 *     if result_code < 0:
 *         raise VorbisException()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyrogg._checkVorbisResult", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":43
 *     return 0
 * 
 * cdef bytes _encodeFilename(filename):             # <<<<<<<<<<<<<<
 *     if isinstance(filename, bytes):
 *         return <bytes>filename
 */

static PyObject *__pyx_f_6pyrogg__encodeFilename(PyObject *__pyx_v_filename) {
  PyObject *__pyx_v_encoding = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("_encodeFilename", 0);

  /* "pyrogg.pyx":44
 * 
 * cdef bytes _encodeFilename(filename):
 *     if isinstance(filename, bytes):             # <<<<<<<<<<<<<<
 *         return <bytes>filename
 *     elif isinstance(filename, unicode):
 */
  __pyx_t_1 = PyBytes_Check(__pyx_v_filename); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":45
 * cdef bytes _encodeFilename(filename):
 *     if isinstance(filename, bytes):
 *         return <bytes>filename             # <<<<<<<<<<<<<<
 *     elif isinstance(filename, unicode):
 *         encoding = sys.getfilesystemencoding()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_filename));
    __pyx_r = ((PyObject*)__pyx_v_filename);
    goto __pyx_L0;

    /* "pyrogg.pyx":44
 * 
 * cdef bytes _encodeFilename(filename):
 *     if isinstance(filename, bytes):             # <<<<<<<<<<<<<<
 *         return <bytes>filename
 *     elif isinstance(filename, unicode):
 */
  }

  /* "pyrogg.pyx":46
 *     if isinstance(filename, bytes):
 *         return <bytes>filename
 *     elif isinstance(filename, unicode):             # <<<<<<<<<<<<<<
 *         encoding = sys.getfilesystemencoding()
 *         if not encoding:
 */
  __pyx_t_2 = PyUnicode_Check(__pyx_v_filename); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":47
 *         return <bytes>filename
 *     elif isinstance(filename, unicode):
 *         encoding = sys.getfilesystemencoding()             # <<<<<<<<<<<<<<
 *         if not encoding:
 *             encoding = 'iso8859-1'  # just give it a try ...
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_6pyrogg_sys, __pyx_n_s_getfilesystemencoding); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 47, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 47, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_encoding = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyrogg.pyx":48
 *     elif isinstance(filename, unicode):
 *         encoding = sys.getfilesystemencoding()
 *         if not encoding:             # <<<<<<<<<<<<<<
 *             encoding = 'iso8859-1'  # just give it a try ...
 *         return (<unicode>filename).encode(encoding)
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
    __pyx_t_2 = ((!__pyx_t_1) != 0);
    if (__pyx_t_2) {

      /* "pyrogg.pyx":49
 *         encoding = sys.getfilesystemencoding()
 *         if not encoding:
 *             encoding = 'iso8859-1'  # just give it a try ...             # <<<<<<<<<<<<<<
 *         return (<unicode>filename).encode(encoding)
 *     else:
 */
      __Pyx_INCREF(__pyx_kp_s_iso8859_1);
      __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_kp_s_iso8859_1);

      /* "pyrogg.pyx":48
 *     elif isinstance(filename, unicode):
 *         encoding = sys.getfilesystemencoding()
 *         if not encoding:             # <<<<<<<<<<<<<<
 *             encoding = 'iso8859-1'  # just give it a try ...
 *         return (<unicode>filename).encode(encoding)
 */
    }

    /* "pyrogg.pyx":50
 *         if not encoding:
 *             encoding = 'iso8859-1'  # just give it a try ...
 *         return (<unicode>filename).encode(encoding)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("expected file path, got %s" % type(filename))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_filename, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_encoding); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_encoding};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_encoding};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 50, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_encoding);
        __Pyx_GIVEREF(__pyx_v_encoding);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_encoding);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 50, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pyrogg.pyx":46
 *     if isinstance(filename, bytes):
 *         return <bytes>filename
 *     elif isinstance(filename, unicode):             # <<<<<<<<<<<<<<
 *         encoding = sys.getfilesystemencoding()
 *         if not encoding:
 */
  }

  /* "pyrogg.pyx":52
 *         return (<unicode>filename).encode(encoding)
 *     else:
 *         raise ValueError("expected file path, got %s" % type(filename))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_expected_file_path_got_s, ((PyObject *)Py_TYPE(__pyx_v_filename))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 52, __pyx_L1_error)
  }

  /* "pyrogg.pyx":43
 *     return 0
 * 
 * cdef bytes _encodeFilename(filename):             # <<<<<<<<<<<<<<
 *     if isinstance(filename, bytes):
 *         return <bytes>filename
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyrogg._encodeFilename", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":59
 *     cdef object filename
 * 
 *     def __cinit__(self, filename):             # <<<<<<<<<<<<<<
 *         cdef stdio.FILE* cfile
 *         cdef int result
 */

/* Python wrapper */
static int __pyx_pw_6pyrogg_10VorbisFile_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6pyrogg_10VorbisFile_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_filename = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_filename,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 59, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_filename = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 59, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.VorbisFile.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_10VorbisFile___cinit__(((struct __pyx_obj_6pyrogg_VorbisFile *)__pyx_v_self), __pyx_v_filename);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6pyrogg_10VorbisFile___cinit__(struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self, PyObject *__pyx_v_filename) {
  FILE *__pyx_v_cfile;
  int __pyx_v_result;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_filename);

  /* "pyrogg.pyx":62
 *         cdef stdio.FILE* cfile
 *         cdef int result
 *         filename = _encodeFilename(filename)             # <<<<<<<<<<<<<<
 *         cfile = stdio.fopen(filename, "r")
 *         if cfile is NULL:
 */
  __pyx_t_1 = __pyx_f_6pyrogg__encodeFilename(__pyx_v_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_filename, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":63
 *         cdef int result
 *         filename = _encodeFilename(filename)
 *         cfile = stdio.fopen(filename, "r")             # <<<<<<<<<<<<<<
 *         if cfile is NULL:
 *             raise IOError("Opening file %r failed" % filename)
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_filename); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 63, __pyx_L1_error)
  __pyx_v_cfile = fopen(__pyx_t_2, ((char const *)"r"));

  /* "pyrogg.pyx":64
 *         filename = _encodeFilename(filename)
 *         cfile = stdio.fopen(filename, "r")
 *         if cfile is NULL:             # <<<<<<<<<<<<<<
 *             raise IOError("Opening file %r failed" % filename)
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 */
  __pyx_t_3 = ((__pyx_v_cfile == NULL) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":65
 *         cfile = stdio.fopen(filename, "r")
 *         if cfile is NULL:
 *             raise IOError("Opening file %r failed" % filename)             # <<<<<<<<<<<<<<
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 *         if result == 0:
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Opening_file_r_failed, __pyx_v_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 65, __pyx_L1_error)

    /* "pyrogg.pyx":64
 *         filename = _encodeFilename(filename)
 *         cfile = stdio.fopen(filename, "r")
 *         if cfile is NULL:             # <<<<<<<<<<<<<<
 *             raise IOError("Opening file %r failed" % filename)
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 */
  }

  /* "pyrogg.pyx":66
 *         if cfile is NULL:
 *             raise IOError("Opening file %r failed" % filename)
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)             # <<<<<<<<<<<<<<
 *         if result == 0:
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 */
  __pyx_v_result = ov_test(__pyx_v_cfile, (&__pyx_v_self->_vorbisfile), NULL, 0);

  /* "pyrogg.pyx":67
 *             raise IOError("Opening file %r failed" % filename)
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 *         if result == 0:             # <<<<<<<<<<<<<<
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 *         if result < 0:
 */
  __pyx_t_3 = ((__pyx_v_result == 0) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":68
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 *         if result == 0:
 *             result = vorbis.ov_test_open(&self._vorbisfile)             # <<<<<<<<<<<<<<
 *         if result < 0:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 */
    __pyx_v_result = ov_test_open((&__pyx_v_self->_vorbisfile));

    /* "pyrogg.pyx":67
 *             raise IOError("Opening file %r failed" % filename)
 *         result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
 *         if result == 0:             # <<<<<<<<<<<<<<
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 *         if result < 0:
 */
  }

  /* "pyrogg.pyx":69
 *         if result == 0:
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 *             if result == vorbis.OV_ENOTVORBIS:
 */
  __pyx_t_3 = ((__pyx_v_result < 0) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":70
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 *         if result < 0:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file             # <<<<<<<<<<<<<<
 *             if result == vorbis.OV_ENOTVORBIS:
 *                 raise VorbisException("%r is not a Vorbis file" % filename)
 */
    ov_clear((&__pyx_v_self->_vorbisfile));

    /* "pyrogg.pyx":71
 *         if result < 0:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 *             if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *                 raise VorbisException("%r is not a Vorbis file" % filename)
 *             raise IOError("Error reading from file %r" % filename)
 */
    __pyx_t_3 = ((__pyx_v_result == OV_ENOTVORBIS) != 0);
    if (__pyx_t_3) {

      /* "pyrogg.pyx":72
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 *             if result == vorbis.OV_ENOTVORBIS:
 *                 raise VorbisException("%r is not a Vorbis file" % filename)             # <<<<<<<<<<<<<<
 *             raise IOError("Error reading from file %r" % filename)
 *         self.filename = filename
 */
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_r_is_not_a_Vorbis_file, __pyx_v_filename); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 72, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 72, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 72, __pyx_L1_error)

      /* "pyrogg.pyx":71
 *         if result < 0:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 *             if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *                 raise VorbisException("%r is not a Vorbis file" % filename)
 *             raise IOError("Error reading from file %r" % filename)
 */
    }

    /* "pyrogg.pyx":73
 *             if result == vorbis.OV_ENOTVORBIS:
 *                 raise VorbisException("%r is not a Vorbis file" % filename)
 *             raise IOError("Error reading from file %r" % filename)             # <<<<<<<<<<<<<<
 *         self.filename = filename
 * 
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Error_reading_from_file_r, __pyx_v_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 73, __pyx_L1_error)

    /* "pyrogg.pyx":69
 *         if result == 0:
 *             result = vorbis.ov_test_open(&self._vorbisfile)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 *             if result == vorbis.OV_ENOTVORBIS:
 */
  }

  /* "pyrogg.pyx":74
 *                 raise VorbisException("%r is not a Vorbis file" % filename)
 *             raise IOError("Error reading from file %r" % filename)
 *         self.filename = filename             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __Pyx_INCREF(__pyx_v_filename);
  __Pyx_GIVEREF(__pyx_v_filename);
  __Pyx_GOTREF(__pyx_v_self->filename);
  __Pyx_DECREF(__pyx_v_self->filename);
  __pyx_v_self->filename = __pyx_v_filename;

  /* "pyrogg.pyx":59
 *     cdef object filename
 * 
 *     def __cinit__(self, filename):             # <<<<<<<<<<<<<<
 *         cdef stdio.FILE* cfile
 *         cdef int result
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyrogg.VorbisFile.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_filename);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":76
 *         self.filename = filename
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.filename is not None:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 */

/* Python wrapper */
static void __pyx_pw_6pyrogg_10VorbisFile_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6pyrogg_10VorbisFile_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_6pyrogg_10VorbisFile_2__dealloc__(((struct __pyx_obj_6pyrogg_VorbisFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6pyrogg_10VorbisFile_2__dealloc__(struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pyrogg.pyx":77
 * 
 *     def __dealloc__(self):
 *         if self.filename is not None:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 * 
 */
  __pyx_t_1 = (__pyx_v_self->filename != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":78
 *     def __dealloc__(self):
 *         if self.filename is not None:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file             # <<<<<<<<<<<<<<
 * 
 * 
 */
    ov_clear((&__pyx_v_self->_vorbisfile));

    /* "pyrogg.pyx":77
 * 
 *     def __dealloc__(self):
 *         if self.filename is not None:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 * 
 */
  }

  /* "pyrogg.pyx":76
 *         self.filename = filename
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.filename is not None:
 *             vorbis.ov_clear(&self._vorbisfile) # closes the file
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_10VorbisFile_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6pyrogg_10VorbisFile_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_10VorbisFile_4__reduce_cython__(((struct __pyx_obj_6pyrogg_VorbisFile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_10VorbisFile_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFile.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_10VorbisFile_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6pyrogg_10VorbisFile_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_10VorbisFile_6__setstate_cython__(((struct __pyx_obj_6pyrogg_VorbisFile *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_10VorbisFile_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFile *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFile.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":94
 * 
 *     @cython.final
 *     cdef _recode(self, void* status, vorbis.OggVorbis_File* vorbisfile,             # <<<<<<<<<<<<<<
 *                  int target_quality):
 *         cdef int error_status, init_steps
 */

static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recode(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, OggVorbis_File *__pyx_v_vorbisfile, int __pyx_v_target_quality) {
  int __pyx_v_error_status;
  float __pyx_v_cquality;
  char *__pyx_v_decbuffer;
  ogg_stream_state __pyx_v_stream_state;
  vorbis_info __pyx_v_vorbis_info;
  vorbis_dsp_state __pyx_v_dsp_state;
  vorbis_block __pyx_v_vorbis_block;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  char const *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  __Pyx_RefNannySetupContext("_recode", 0);

  /* "pyrogg.pyx":98
 *         cdef int error_status, init_steps
 *         cdef float cquality
 *         cdef char* decbuffer = NULL             # <<<<<<<<<<<<<<
 *         cdef ogg.ogg_stream_state stream_state
 *         cdef vorbis.vorbis_info vorbis_info
 */
  __pyx_v_decbuffer = NULL;

  /* "pyrogg.pyx":104
 *         cdef vorbis.vorbis_block vorbis_block
 * 
 *         if target_quality > 10:             # <<<<<<<<<<<<<<
 *             cquality = 1.0
 *         elif target_quality < -1:
 */
  __pyx_t_1 = ((__pyx_v_target_quality > 10) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":105
 * 
 *         if target_quality > 10:
 *             cquality = 1.0             # <<<<<<<<<<<<<<
 *         elif target_quality < -1:
 *             cquality = -0.1
 */
    __pyx_v_cquality = 1.0;

    /* "pyrogg.pyx":104
 *         cdef vorbis.vorbis_block vorbis_block
 * 
 *         if target_quality > 10:             # <<<<<<<<<<<<<<
 *             cquality = 1.0
 *         elif target_quality < -1:
 */
    goto __pyx_L3;
  }

  /* "pyrogg.pyx":106
 *         if target_quality > 10:
 *             cquality = 1.0
 *         elif target_quality < -1:             # <<<<<<<<<<<<<<
 *             cquality = -0.1
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_target_quality < -1L) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":107
 *             cquality = 1.0
 *         elif target_quality < -1:
 *             cquality = -0.1             # <<<<<<<<<<<<<<
 *         else:
 *             cquality = target_quality / 10.0
 */
    __pyx_v_cquality = -0.1;

    /* "pyrogg.pyx":106
 *         if target_quality > 10:
 *             cquality = 1.0
 *         elif target_quality < -1:             # <<<<<<<<<<<<<<
 *             cquality = -0.1
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyrogg.pyx":109
 *             cquality = -0.1
 *         else:
 *             cquality = target_quality / 10.0             # <<<<<<<<<<<<<<
 * 
 *         # encoding setup
 */
  /*else*/ {
    __pyx_v_cquality = (__pyx_v_target_quality / 10.0);
  }
  __pyx_L3:;

  /* "pyrogg.pyx":112
 * 
 *         # encoding setup
 *         vorbis.vorbis_info_init(&vorbis_info)             # <<<<<<<<<<<<<<
 *         error_status = vorbis.vorbis_encode_init_vbr(
 *             &vorbis_info, vorbisfile.vi.channels,
 */
  vorbis_info_init((&__pyx_v_vorbis_info));

  /* "pyrogg.pyx":113
 *         # encoding setup
 *         vorbis.vorbis_info_init(&vorbis_info)
 *         error_status = vorbis.vorbis_encode_init_vbr(             # <<<<<<<<<<<<<<
 *             &vorbis_info, vorbisfile.vi.channels,
 *             vorbisfile.vi.rate, cquality)
 */
  __pyx_v_error_status = vorbis_encode_init_vbr((&__pyx_v_vorbis_info), __pyx_v_vorbisfile->vi->channels, __pyx_v_vorbisfile->vi->rate, __pyx_v_cquality);

  /* "pyrogg.pyx":116
 *             &vorbis_info, vorbisfile.vi.channels,
 *             vorbisfile.vi.rate, cquality)
 *         if not error_status:             # <<<<<<<<<<<<<<
 *             error_status = _initVorbisEncoder(
 *                 &stream_state, &dsp_state, &vorbis_info,
 */
  __pyx_t_1 = ((!(__pyx_v_error_status != 0)) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":117
 *             vorbisfile.vi.rate, cquality)
 *         if not error_status:
 *             error_status = _initVorbisEncoder(             # <<<<<<<<<<<<<<
 *                 &stream_state, &dsp_state, &vorbis_info,
 *                 # use arbitrary but predictable serial number
 */
    __pyx_v_error_status = __pyx_f_6pyrogg__initVorbisEncoder((&__pyx_v_stream_state), (&__pyx_v_dsp_state), (&__pyx_v_vorbis_info), (__pyx_v_vorbisfile->current_serialno | ((int)((__pyx_v_cquality * 10.0) + 2.0))));

    /* "pyrogg.pyx":116
 *             &vorbis_info, vorbisfile.vi.channels,
 *             vorbisfile.vi.rate, cquality)
 *         if not error_status:             # <<<<<<<<<<<<<<
 *             error_status = _initVorbisEncoder(
 *                 &stream_state, &dsp_state, &vorbis_info,
 */
  }

  /* "pyrogg.pyx":121
 *                 # use arbitrary but predictable serial number
 *                 vorbisfile.current_serialno | <int>(cquality*10+2))
 *         if not error_status:             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_block_init(&dsp_state, &vorbis_block)
 *             error_status = self._writeVorbisHeader(
 */
  __pyx_t_1 = ((!(__pyx_v_error_status != 0)) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":122
 *                 vorbisfile.current_serialno | <int>(cquality*10+2))
 *         if not error_status:
 *             vorbis.vorbis_block_init(&dsp_state, &vorbis_block)             # <<<<<<<<<<<<<<
 *             error_status = self._writeVorbisHeader(
 *                 status, &stream_state, &dsp_state, vorbisfile.vc)
 */
    vorbis_block_init((&__pyx_v_dsp_state), (&__pyx_v_vorbis_block));

    /* "pyrogg.pyx":123
 *         if not error_status:
 *             vorbis.vorbis_block_init(&dsp_state, &vorbis_block)
 *             error_status = self._writeVorbisHeader(             # <<<<<<<<<<<<<<
 *                 status, &stream_state, &dsp_state, vorbisfile.vc)
 * 
 */
    __pyx_v_error_status = __pyx_f_6pyrogg_14_VorbisRecoder__writeVorbisHeader(__pyx_v_self, __pyx_v_status, (&__pyx_v_stream_state), (&__pyx_v_dsp_state), __pyx_v_vorbisfile->vc);

    /* "pyrogg.pyx":121
 *                 # use arbitrary but predictable serial number
 *                 vorbisfile.current_serialno | <int>(cquality*10+2))
 *         if not error_status:             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_block_init(&dsp_state, &vorbis_block)
 *             error_status = self._writeVorbisHeader(
 */
  }

  /* "pyrogg.pyx":126
 *                 status, &stream_state, &dsp_state, vorbisfile.vc)
 * 
 *         t = time()             # <<<<<<<<<<<<<<
 *         try:
 *             if not error_status:
 */
  __Pyx_INCREF(__pyx_v_6pyrogg_time);
  __pyx_t_3 = __pyx_v_6pyrogg_time; __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_t = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyrogg.pyx":127
 * 
 *         t = time()
 *         try:             # <<<<<<<<<<<<<<
 *             if not error_status:
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 */
  /*try:*/ {

    /* "pyrogg.pyx":128
 *         t = time()
 *         try:
 *             if not error_status:             # <<<<<<<<<<<<<<
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 *                 if decbuffer is NULL:
 */
    __pyx_t_1 = ((!(__pyx_v_error_status != 0)) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":129
 *         try:
 *             if not error_status:
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)             # <<<<<<<<<<<<<<
 *                 if decbuffer is NULL:
 *                     raise MemoryError()
 */
      __pyx_v_decbuffer = ((char *)PyMem_Malloc(0x200000));

      /* "pyrogg.pyx":130
 *             if not error_status:
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 *                 if decbuffer is NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError()
 *                 self._recodeParallel(
 */
      __pyx_t_1 = ((__pyx_v_decbuffer == NULL) != 0);
      if (__pyx_t_1) {

        /* "pyrogg.pyx":131
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 *                 if decbuffer is NULL:
 *                     raise MemoryError()             # <<<<<<<<<<<<<<
 *                 self._recodeParallel(
 *                     status, decbuffer, vorbisfile,
 */
        PyErr_NoMemory(); __PYX_ERR(0, 131, __pyx_L7_error)

        /* "pyrogg.pyx":130
 *             if not error_status:
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 *                 if decbuffer is NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError()
 *                 self._recodeParallel(
 */
      }

      /* "pyrogg.pyx":132
 *                 if decbuffer is NULL:
 *                     raise MemoryError()
 *                 self._recodeParallel(             # <<<<<<<<<<<<<<
 *                     status, decbuffer, vorbisfile,
 *                     dsp_state, vorbis_block, stream_state)
 */
      __pyx_t_2 = __pyx_f_6pyrogg_14_VorbisRecoder__recodeParallel(__pyx_v_self, __pyx_v_status, __pyx_v_decbuffer, __pyx_v_vorbisfile, __pyx_v_dsp_state, __pyx_v_vorbis_block, __pyx_v_stream_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L7_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyrogg.pyx":128
 *         t = time()
 *         try:
 *             if not error_status:             # <<<<<<<<<<<<<<
 *                 decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
 *                 if decbuffer is NULL:
 */
    }
  }

  /* "pyrogg.pyx":136
 *                     dsp_state, vorbis_block, stream_state)
 *         finally:
 *             ogg.ogg_stream_clear(&stream_state)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 */
  /*finally:*/ {
    /*normal exit:*/{
      ogg_stream_clear((&__pyx_v_stream_state));

      /* "pyrogg.pyx":137
 *         finally:
 *             ogg.ogg_stream_clear(&stream_state)
 *             vorbis.vorbis_block_clear(&vorbis_block)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)
 */
      vorbis_block_clear((&__pyx_v_vorbis_block));

      /* "pyrogg.pyx":138
 *             ogg.ogg_stream_clear(&stream_state)
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)
 */
      vorbis_dsp_clear((&__pyx_v_dsp_state));

      /* "pyrogg.pyx":139
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)             # <<<<<<<<<<<<<<
 *             cpython.mem.PyMem_Free(decbuffer)
 *             vorbis.ov_clear(vorbisfile) # closes the input file
 */
      vorbis_info_clear((&__pyx_v_vorbis_info));

      /* "pyrogg.pyx":140
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(vorbisfile) # closes the input file
 *         t = time() - t
 */
      PyMem_Free(__pyx_v_decbuffer);

      /* "pyrogg.pyx":141
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)
 *             vorbis.ov_clear(vorbisfile) # closes the input file             # <<<<<<<<<<<<<<
 *         t = time() - t
 * 
 */
      ov_clear(__pyx_v_vorbisfile);
      goto __pyx_L8;
    }
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __pyx_L7_error:;
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __Pyx_PyThreadState_assign
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0)) __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __pyx_t_5 = __pyx_lineno; __pyx_t_6 = __pyx_clineno; __pyx_t_7 = __pyx_filename;
      {

        /* "pyrogg.pyx":136
 *                     dsp_state, vorbis_block, stream_state)
 *         finally:
 *             ogg.ogg_stream_clear(&stream_state)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 */
        ogg_stream_clear((&__pyx_v_stream_state));

        /* "pyrogg.pyx":137
 *         finally:
 *             ogg.ogg_stream_clear(&stream_state)
 *             vorbis.vorbis_block_clear(&vorbis_block)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)
 */
        vorbis_block_clear((&__pyx_v_vorbis_block));

        /* "pyrogg.pyx":138
 *             ogg.ogg_stream_clear(&stream_state)
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)             # <<<<<<<<<<<<<<
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)
 */
        vorbis_dsp_clear((&__pyx_v_dsp_state));

        /* "pyrogg.pyx":139
 *             vorbis.vorbis_block_clear(&vorbis_block)
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)             # <<<<<<<<<<<<<<
 *             cpython.mem.PyMem_Free(decbuffer)
 *             vorbis.ov_clear(vorbisfile) # closes the input file
 */
        vorbis_info_clear((&__pyx_v_vorbis_info));

        /* "pyrogg.pyx":140
 *             vorbis.vorbis_dsp_clear(&dsp_state)
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(vorbisfile) # closes the input file
 *         t = time() - t
 */
        PyMem_Free(__pyx_v_decbuffer);

        /* "pyrogg.pyx":141
 *             vorbis.vorbis_info_clear(&vorbis_info)
 *             cpython.mem.PyMem_Free(decbuffer)
 *             vorbis.ov_clear(vorbisfile) # closes the input file             # <<<<<<<<<<<<<<
 *         t = time() - t
 * 
 */
        ov_clear(__pyx_v_vorbisfile);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      }
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ErrRestore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_6; __pyx_filename = __pyx_t_7;
      goto __pyx_L1_error;
    }
    __pyx_L8:;
  }

  /* "pyrogg.pyx":142
 *             cpython.mem.PyMem_Free(decbuffer)
 *             vorbis.ov_clear(vorbisfile) # closes the input file
 *         t = time() - t             # <<<<<<<<<<<<<<
 * 
 *         if error_status:
 */
  __Pyx_INCREF(__pyx_v_6pyrogg_time);
  __pyx_t_3 = __pyx_v_6pyrogg_time; __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_2, __pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_t, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyrogg.pyx":144
 *         t = time() - t
 * 
 *         if error_status:             # <<<<<<<<<<<<<<
 *             raise VorbisException(
 *                 "Unknown error %d occurred while preparing recoder" %
 */
  __pyx_t_1 = (__pyx_v_error_status != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":145
 * 
 *         if error_status:
 *             raise VorbisException(             # <<<<<<<<<<<<<<
 *                 "Unknown error %d occurred while preparing recoder" %
 *                 error_status)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "pyrogg.pyx":147
 *             raise VorbisException(
 *                 "Unknown error %d occurred while preparing recoder" %
 *                 error_status)             # <<<<<<<<<<<<<<
 * 
 *         return t
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_error_status); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "pyrogg.pyx":146
 *         if error_status:
 *             raise VorbisException(
 *                 "Unknown error %d occurred while preparing recoder" %             # <<<<<<<<<<<<<<
 *                 error_status)
 * 
 */
    __pyx_t_14 = __Pyx_PyString_Format(__pyx_kp_s_Unknown_error_d_occurred_while_p, __pyx_t_4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_14); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_14};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_14};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      } else
      #endif
      {
        __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_14);
        PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_t_14);
        __pyx_t_14 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_15, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 145, __pyx_L1_error)

    /* "pyrogg.pyx":144
 *         t = time() - t
 * 
 *         if error_status:             # <<<<<<<<<<<<<<
 *             raise VorbisException(
 *                 "Unknown error %d occurred while preparing recoder" %
 */
  }

  /* "pyrogg.pyx":149
 *                 error_status)
 * 
 *         return t             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "pyrogg.pyx":94
 * 
 *     @cython.final
 *     cdef _recode(self, void* status, vorbis.OggVorbis_File* vorbisfile,             # <<<<<<<<<<<<<<
 *                  int target_quality):
 *         cdef int error_status, init_steps
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("pyrogg._VorbisRecoder._recode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":152
 * 
 *     @cython.final
 *     cdef _recodeParallel(self, void* status, char* decbuffer,             # <<<<<<<<<<<<<<
 *                          vorbis.OggVorbis_File* vorbisfile,
 *                          vorbis.vorbis_dsp_state dsp_state,
 */

static PyObject *__pyx_f_6pyrogg_14_VorbisRecoder__recodeParallel(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, char *__pyx_v_decbuffer, OggVorbis_File *__pyx_v_vorbisfile, vorbis_dsp_state __pyx_v_dsp_state, vorbis_block __pyx_v_vorbis_block, ogg_stream_state __pyx_v_stream_state) {
  long __pyx_v_read_status;
  long __pyx_v_values_read;
  int __pyx_v_i;
  int __pyx_v_eos;
  int __pyx_v_buffer_part;
  int __pyx_v_current_section;
  struct __pyx_t_6pyrogg_communication __pyx_v_com;
  float **__pyx_v_encbuffer;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("_recodeParallel", 0);

  /* "pyrogg.pyx":162
 *         cdef communication com   # thread communication
 * 
 *         com.read_status[0] = vorbis.ov_read(             # <<<<<<<<<<<<<<
 *             vorbisfile, decbuffer, READ_BUFFER_SIZE,
 *             0, 2, 1, &current_section)
 */
  (__pyx_v_com.read_status[0]) = ov_read(__pyx_v_vorbisfile, __pyx_v_decbuffer, 0x100000, 0, 2, 1, (&__pyx_v_current_section));

  /* "pyrogg.pyx":165
 *             vorbisfile, decbuffer, READ_BUFFER_SIZE,
 *             0, 2, 1, &current_section)
 *         _checkVorbisResult(com.read_status[0])             # <<<<<<<<<<<<<<
 * 
 *         com.done = 0
 */
  __pyx_t_1 = __pyx_f_6pyrogg__checkVorbisResult((__pyx_v_com.read_status[0])); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 165, __pyx_L1_error)

  /* "pyrogg.pyx":167
 *         _checkVorbisResult(com.read_status[0])
 * 
 *         com.done = 0             # <<<<<<<<<<<<<<
 *         com.buffer_ready = 0
 * 
 */
  __pyx_v_com.done = 0;

  /* "pyrogg.pyx":168
 * 
 *         com.done = 0
 *         com.buffer_ready = 0             # <<<<<<<<<<<<<<
 * 
 *         with nogil, parallel.parallel(num_threads=2):
 */
  __pyx_v_com.buffer_ready = 0;

  /* "pyrogg.pyx":170
 *         com.buffer_ready = 0
 * 
 *         with nogil, parallel.parallel(num_threads=2):             # <<<<<<<<<<<<<<
 *             while True:
 *                 # using prange() loop to synchronise
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {
        {
            #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
                #undef likely
                #undef unlikely
                #define likely(x)   (x)
                #define unlikely(x) (x)
            #endif
            #ifdef _OPENMP
            #pragma omp parallel  private(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4) num_threads(2)
            #endif /* _OPENMP */
            {

                /* "pyrogg.pyx":171
 * 
 *         with nogil, parallel.parallel(num_threads=2):
 *             while True:             # <<<<<<<<<<<<<<
 *                 # using prange() loop to synchronise
 *                 for i in parallel.prange(2):
 */
                while (1) {

                  /* "pyrogg.pyx":173
 *             while True:
 *                 # using prange() loop to synchronise
 *                 for i in parallel.prange(2):             # <<<<<<<<<<<<<<
 *                     if i == 1:
 *                         # reader thread
 */
                  if (1 == 0) abort();
                  {
                      __pyx_t_3 = (2 - 0 + 1 - 1/abs(1)) / 1;
                      if (__pyx_t_3 > 0)
                      {
                          #ifdef _OPENMP
                          #pragma omp for lastprivate(__pyx_v_buffer_part) lastprivate(__pyx_v_encbuffer) lastprivate(__pyx_v_eos) firstprivate(__pyx_v_i) lastprivate(__pyx_v_i) lastprivate(__pyx_v_read_status) lastprivate(__pyx_v_values_read)
                          #endif /* _OPENMP */
                          for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_3; __pyx_t_2++){
                              {
                                  __pyx_v_i = (int)(0 + 1 * __pyx_t_2);
                                  /* Initialize private variables to invalid values */
                                  __pyx_v_buffer_part = ((int)0xbad0bad0);
                                  __pyx_v_encbuffer = ((float **)1);
                                  __pyx_v_eos = ((int)0xbad0bad0);
                                  __pyx_v_read_status = ((long)0xbad0bad0);
                                  __pyx_v_values_read = ((long)0xbad0bad0);

                                  /* "pyrogg.pyx":174
 *                 # using prange() loop to synchronise
 *                 for i in parallel.prange(2):
 *                     if i == 1:             # <<<<<<<<<<<<<<
 *                         # reader thread
 *                         buffer_part = 0 if com.buffer_ready == 1 else 1
 */
                                  __pyx_t_4 = ((__pyx_v_i == 1) != 0);
                                  if (__pyx_t_4) {

                                    /* "pyrogg.pyx":176
 *                     if i == 1:
 *                         # reader thread
 *                         buffer_part = 0 if com.buffer_ready == 1 else 1             # <<<<<<<<<<<<<<
 *                         read_status = vorbis.ov_read(
 *                             vorbisfile,
 */
                                    if (((__pyx_v_com.buffer_ready == 1) != 0)) {
                                      __pyx_t_1 = 0;
                                    } else {
                                      __pyx_t_1 = 1;
                                    }
                                    __pyx_v_buffer_part = __pyx_t_1;

                                    /* "pyrogg.pyx":177
 *                         # reader thread
 *                         buffer_part = 0 if com.buffer_ready == 1 else 1
 *                         read_status = vorbis.ov_read(             # <<<<<<<<<<<<<<
 *                             vorbisfile,
 *                             decbuffer + (READ_BUFFER_SIZE * buffer_part),
 */
                                    __pyx_v_read_status = ov_read(__pyx_v_vorbisfile, (__pyx_v_decbuffer + (0x100000 * __pyx_v_buffer_part)), 0x100000, 0, 2, 1, (&__pyx_v_current_section));

                                    /* "pyrogg.pyx":182
 *                             READ_BUFFER_SIZE,
 *                             0, 2, 1, &current_section)
 *                         com.read_status[buffer_part] = read_status             # <<<<<<<<<<<<<<
 *                         com.next_buffer_ready = buffer_part
 *                     else:
 */
                                    (__pyx_v_com.read_status[__pyx_v_buffer_part]) = __pyx_v_read_status;

                                    /* "pyrogg.pyx":183
 *                             0, 2, 1, &current_section)
 *                         com.read_status[buffer_part] = read_status
 *                         com.next_buffer_ready = buffer_part             # <<<<<<<<<<<<<<
 *                     else:
 *                         # writer thread
 */
                                    __pyx_v_com.next_buffer_ready = __pyx_v_buffer_part;

                                    /* "pyrogg.pyx":174
 *                 # using prange() loop to synchronise
 *                 for i in parallel.prange(2):
 *                     if i == 1:             # <<<<<<<<<<<<<<
 *                         # reader thread
 *                         buffer_part = 0 if com.buffer_ready == 1 else 1
 */
                                    goto __pyx_L16;
                                  }

                                  /* "pyrogg.pyx":186
 *                     else:
 *                         # writer thread
 *                         buffer_part = com.buffer_ready             # <<<<<<<<<<<<<<
 *                         read_status = com.read_status[buffer_part]
 *                         encbuffer = vorbis.vorbis_analysis_buffer(
 */
                                  /*else*/ {
                                    __pyx_t_1 = __pyx_v_com.buffer_ready;
                                    __pyx_v_buffer_part = __pyx_t_1;

                                    /* "pyrogg.pyx":187
 *                         # writer thread
 *                         buffer_part = com.buffer_ready
 *                         read_status = com.read_status[buffer_part]             # <<<<<<<<<<<<<<
 *                         encbuffer = vorbis.vorbis_analysis_buffer(
 *                             &dsp_state, READ_BUFFER_SIZE/4)
 */
                                    __pyx_v_read_status = (__pyx_v_com.read_status[__pyx_v_buffer_part]);

                                    /* "pyrogg.pyx":188
 *                         buffer_part = com.buffer_ready
 *                         read_status = com.read_status[buffer_part]
 *                         encbuffer = vorbis.vorbis_analysis_buffer(             # <<<<<<<<<<<<<<
 *                             &dsp_state, READ_BUFFER_SIZE/4)
 *                         if read_status > 0:
 */
                                    __pyx_v_encbuffer = vorbis_analysis_buffer((&__pyx_v_dsp_state), 0x40000);

                                    /* "pyrogg.pyx":190
 *                         encbuffer = vorbis.vorbis_analysis_buffer(
 *                             &dsp_state, READ_BUFFER_SIZE/4)
 *                         if read_status > 0:             # <<<<<<<<<<<<<<
 *                             values_read = read_status / 4
 *                             _splitStereo(
 */
                                    __pyx_t_4 = ((__pyx_v_read_status > 0) != 0);
                                    if (__pyx_t_4) {

                                      /* "pyrogg.pyx":191
 *                             &dsp_state, READ_BUFFER_SIZE/4)
 *                         if read_status > 0:
 *                             values_read = read_status / 4             # <<<<<<<<<<<<<<
 *                             _splitStereo(
 *                                 decbuffer + (READ_BUFFER_SIZE * buffer_part),
 */
                                      __pyx_v_values_read = __Pyx_div_long(__pyx_v_read_status, 4);

                                      /* "pyrogg.pyx":192
 *                         if read_status > 0:
 *                             values_read = read_status / 4
 *                             _splitStereo(             # <<<<<<<<<<<<<<
 *                                 decbuffer + (READ_BUFFER_SIZE * buffer_part),
 *                                 encbuffer[0], encbuffer[1], values_read)
 */
                                      __pyx_f_6pyrogg__splitStereo((__pyx_v_decbuffer + (0x100000 * __pyx_v_buffer_part)), (__pyx_v_encbuffer[0]), (__pyx_v_encbuffer[1]), __pyx_v_values_read);

                                      /* "pyrogg.pyx":190
 *                         encbuffer = vorbis.vorbis_analysis_buffer(
 *                             &dsp_state, READ_BUFFER_SIZE/4)
 *                         if read_status > 0:             # <<<<<<<<<<<<<<
 *                             values_read = read_status / 4
 *                             _splitStereo(
 */
                                      goto __pyx_L17;
                                    }

                                    /* "pyrogg.pyx":196
 *                                 encbuffer[0], encbuffer[1], values_read)
 *                         else:
 *                             values_read = 0             # <<<<<<<<<<<<<<
 *                         vorbis.vorbis_analysis_wrote(&dsp_state, values_read)
 * 
 */
                                    /*else*/ {
                                      __pyx_v_values_read = 0;
                                    }
                                    __pyx_L17:;

                                    /* "pyrogg.pyx":197
 *                         else:
 *                             values_read = 0
 *                         vorbis.vorbis_analysis_wrote(&dsp_state, values_read)             # <<<<<<<<<<<<<<
 * 
 *                         eos = self._encodeVorbisBlocks(
 */
                                    vorbis_analysis_wrote((&__pyx_v_dsp_state), __pyx_v_values_read);

                                    /* "pyrogg.pyx":199
 *                         vorbis.vorbis_analysis_wrote(&dsp_state, values_read)
 * 
 *                         eos = self._encodeVorbisBlocks(             # <<<<<<<<<<<<<<
 *                             status, &stream_state, &dsp_state,
 *                             &vorbis_block)
 */
                                    __pyx_v_eos = __pyx_f_6pyrogg_14_VorbisRecoder__encodeVorbisBlocks(__pyx_v_self, __pyx_v_status, (&__pyx_v_stream_state), (&__pyx_v_dsp_state), (&__pyx_v_vorbis_block));

                                    /* "pyrogg.pyx":202
 *                             status, &stream_state, &dsp_state,
 *                             &vorbis_block)
 *                         if eos:             # <<<<<<<<<<<<<<
 *                             com.done = 1
 *                 if com.done:
 */
                                    __pyx_t_4 = (__pyx_v_eos != 0);
                                    if (__pyx_t_4) {

                                      /* "pyrogg.pyx":203
 *                             &vorbis_block)
 *                         if eos:
 *                             com.done = 1             # <<<<<<<<<<<<<<
 *                 if com.done:
 *                     break
 */
                                      __pyx_v_com.done = 1;

                                      /* "pyrogg.pyx":202
 *                             status, &stream_state, &dsp_state,
 *                             &vorbis_block)
 *                         if eos:             # <<<<<<<<<<<<<<
 *                             com.done = 1
 *                 if com.done:
 */
                                    }
                                  }
                                  __pyx_L16:;
                              }
                          }
                      }
                  }

                  /* "pyrogg.pyx":204
 *                         if eos:
 *                             com.done = 1
 *                 if com.done:             # <<<<<<<<<<<<<<
 *                     break
 *                 # switch buffers when both are done
 */
                  __pyx_t_4 = (__pyx_v_com.done != 0);
                  if (__pyx_t_4) {

                    /* "pyrogg.pyx":205
 *                             com.done = 1
 *                 if com.done:
 *                     break             # <<<<<<<<<<<<<<
 *                 # switch buffers when both are done
 *                 com.buffer_ready = com.next_buffer_ready
 */
                    goto __pyx_L11_break;

                    /* "pyrogg.pyx":204
 *                         if eos:
 *                             com.done = 1
 *                 if com.done:             # <<<<<<<<<<<<<<
 *                     break
 *                 # switch buffers when both are done
 */
                  }

                  /* "pyrogg.pyx":207
 *                     break
 *                 # switch buffers when both are done
 *                 com.buffer_ready = com.next_buffer_ready             # <<<<<<<<<<<<<<
 * 
 *         read_status = com.read_status[com.buffer_ready]
 */
                  __pyx_t_1 = __pyx_v_com.next_buffer_ready;
                  __pyx_v_com.buffer_ready = __pyx_t_1;
                }
                __pyx_L11_break:;
            }
        }
        #if ((defined(__APPLE__) || defined(__OSX__)) && (defined(__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))))
            #undef likely
            #undef unlikely
            #define likely(x)   __builtin_expect(!!(x), 1)
            #define unlikely(x) __builtin_expect(!!(x), 0)
        #endif
      }

      /* "pyrogg.pyx":170
 *         com.buffer_ready = 0
 * 
 *         with nogil, parallel.parallel(num_threads=2):             # <<<<<<<<<<<<<<
 *             while True:
 *                 # using prange() loop to synchronise
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyrogg.pyx":209
 *                 com.buffer_ready = com.next_buffer_ready
 * 
 *         read_status = com.read_status[com.buffer_ready]             # <<<<<<<<<<<<<<
 *         if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:
 *             raise VorbisDecodeException("lost sync")
 */
  __pyx_v_read_status = (__pyx_v_com.read_status[__pyx_v_com.buffer_ready]);

  /* "pyrogg.pyx":210
 * 
 *         read_status = com.read_status[com.buffer_ready]
 *         if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:             # <<<<<<<<<<<<<<
 *             raise VorbisDecodeException("lost sync")
 * 
 */
  switch (__pyx_v_read_status) {
    case OV_HOLE:
    case OV_EBADLINK:

    /* "pyrogg.pyx":211
 *         read_status = com.read_status[com.buffer_ready]
 *         if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:
 *             raise VorbisDecodeException("lost sync")             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisDecodeException); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 211, __pyx_L1_error)

    /* "pyrogg.pyx":210
 * 
 *         read_status = com.read_status[com.buffer_ready]
 *         if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:             # <<<<<<<<<<<<<<
 *             raise VorbisDecodeException("lost sync")
 * 
 */
    break;
    default: break;
  }

  /* "pyrogg.pyx":152
 * 
 *     @cython.final
 *     cdef _recodeParallel(self, void* status, char* decbuffer,             # <<<<<<<<<<<<<<
 *                          vorbis.OggVorbis_File* vorbisfile,
 *                          vorbis.vorbis_dsp_state dsp_state,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyrogg._VorbisRecoder._recodeParallel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":214
 * 
 *     @cython.final
 *     cdef int _writeVorbisHeader(self, void* status,             # <<<<<<<<<<<<<<
 *                                  ogg.ogg_stream_state* stream_state,
 *                                  vorbis.vorbis_dsp_state* dsp_state,
 */

static int __pyx_f_6pyrogg_14_VorbisRecoder__writeVorbisHeader(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state, vorbis_comment *__pyx_v_comment) {
  ogg_packet __pyx_v_header;
  ogg_packet __pyx_v_header_comm;
  ogg_packet __pyx_v_header_code;
  ogg_page __pyx_v_header_page;
  int __pyx_v_result;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_writeVorbisHeader", 0);

  /* "pyrogg.pyx":223
 *         cdef ogg.ogg_page   header_page
 *         cdef int result
 *         result = vorbis.vorbis_analysis_headerout(             # <<<<<<<<<<<<<<
 *             dsp_state, comment,
 *             &header, &header_comm, &header_code)
 */
  __pyx_v_result = vorbis_analysis_headerout(__pyx_v_dsp_state, __pyx_v_comment, (&__pyx_v_header), (&__pyx_v_header_comm), (&__pyx_v_header_code));

  /* "pyrogg.pyx":226
 *             dsp_state, comment,
 *             &header, &header_comm, &header_code)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             return result
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 */
  __pyx_t_1 = ((__pyx_v_result < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":227
 *             &header, &header_comm, &header_code)
 *         if result < 0:
 *             return result             # <<<<<<<<<<<<<<
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 *             return -1
 */
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "pyrogg.pyx":226
 *             dsp_state, comment,
 *             &header, &header_comm, &header_code)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             return result
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 */
  }

  /* "pyrogg.pyx":228
 *         if result < 0:
 *             return result
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 */
  __pyx_t_1 = ((ogg_stream_packetin(__pyx_v_stream_state, (&__pyx_v_header)) < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":229
 *             return result
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 *             return -1             # <<<<<<<<<<<<<<
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 *             return -1
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyrogg.pyx":228
 *         if result < 0:
 *             return result
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 */
  }

  /* "pyrogg.pyx":230
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:
 */
  __pyx_t_1 = ((ogg_stream_packetin(__pyx_v_stream_state, (&__pyx_v_header_comm)) < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":231
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 *             return -1             # <<<<<<<<<<<<<<
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:
 *             return -1
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyrogg.pyx":230
 *         if ogg.ogg_stream_packetin(stream_state, &header) < 0:
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:
 */
  }

  /* "pyrogg.pyx":232
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
  __pyx_t_1 = ((ogg_stream_packetin(__pyx_v_stream_state, (&__pyx_v_header_code)) < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":233
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *         result = ogg.ogg_stream_flush(stream_state, &header_page)
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyrogg.pyx":232
 *         if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
 *             return -1
 *         if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
  }

  /* "pyrogg.pyx":235
 *             return -1
 * 
 *         result = ogg.ogg_stream_flush(stream_state, &header_page)             # <<<<<<<<<<<<<<
 *         while result > 0:
 *             result = self._write(status, &header_page)
 */
  __pyx_v_result = ogg_stream_flush(__pyx_v_stream_state, (&__pyx_v_header_page));

  /* "pyrogg.pyx":236
 * 
 *         result = ogg.ogg_stream_flush(stream_state, &header_page)
 *         while result > 0:             # <<<<<<<<<<<<<<
 *             result = self._write(status, &header_page)
 *             if result < 0:
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_result > 0) != 0);
    if (!__pyx_t_1) break;

    /* "pyrogg.pyx":237
 *         result = ogg.ogg_stream_flush(stream_state, &header_page)
 *         while result > 0:
 *             result = self._write(status, &header_page)             # <<<<<<<<<<<<<<
 *             if result < 0:
 *                 return result
 */
    __pyx_v_result = __pyx_v_self->_write(__pyx_v_status, (&__pyx_v_header_page));

    /* "pyrogg.pyx":238
 *         while result > 0:
 *             result = self._write(status, &header_page)
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 return result
 *             result = ogg.ogg_stream_flush(stream_state, &header_page)
 */
    __pyx_t_1 = ((__pyx_v_result < 0) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":239
 *             result = self._write(status, &header_page)
 *             if result < 0:
 *                 return result             # <<<<<<<<<<<<<<
 *             result = ogg.ogg_stream_flush(stream_state, &header_page)
 *         return 0
 */
      __pyx_r = __pyx_v_result;
      goto __pyx_L0;

      /* "pyrogg.pyx":238
 *         while result > 0:
 *             result = self._write(status, &header_page)
 *             if result < 0:             # <<<<<<<<<<<<<<
 *                 return result
 *             result = ogg.ogg_stream_flush(stream_state, &header_page)
 */
    }

    /* "pyrogg.pyx":240
 *             if result < 0:
 *                 return result
 *             result = ogg.ogg_stream_flush(stream_state, &header_page)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    __pyx_v_result = ogg_stream_flush(__pyx_v_stream_state, (&__pyx_v_header_page));
  }

  /* "pyrogg.pyx":241
 *                 return result
 *             result = ogg.ogg_stream_flush(stream_state, &header_page)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyrogg.pyx":214
 * 
 *     @cython.final
 *     cdef int _writeVorbisHeader(self, void* status,             # <<<<<<<<<<<<<<
 *                                  ogg.ogg_stream_state* stream_state,
 *                                  vorbis.vorbis_dsp_state* dsp_state,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":244
 * 
 *     @cython.final
 *     cdef int _encodeVorbisBlocks(self, void* status,             # <<<<<<<<<<<<<<
 *                                  ogg.ogg_stream_state* stream_state,
 *                                  vorbis.vorbis_dsp_state* dsp_state,
 */

static int __pyx_f_6pyrogg_14_VorbisRecoder__encodeVorbisBlocks(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state, vorbis_block *__pyx_v_vorbis_block) {
  int __pyx_v_eos;
  int __pyx_v_error_status;
  int __pyx_r;
  int __pyx_t_1;

  /* "pyrogg.pyx":249
 *                                  vorbis.vorbis_block* vorbis_block) nogil:
 *         cdef int eos, error_status
 *         eos = 0             # <<<<<<<<<<<<<<
 *         error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         while error_status == 1:
 */
  __pyx_v_eos = 0;

  /* "pyrogg.pyx":250
 *         cdef int eos, error_status
 *         eos = 0
 *         error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)             # <<<<<<<<<<<<<<
 *         while error_status == 1:
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
 */
  __pyx_v_error_status = vorbis_analysis_blockout(__pyx_v_dsp_state, __pyx_v_vorbis_block);

  /* "pyrogg.pyx":251
 *         eos = 0
 *         error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         while error_status == 1:             # <<<<<<<<<<<<<<
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
 *             if error_status < 0:
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_error_status == 1) != 0);
    if (!__pyx_t_1) break;

    /* "pyrogg.pyx":252
 *         error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         while error_status == 1:
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)             # <<<<<<<<<<<<<<
 *             if error_status < 0:
 *                 return error_status
 */
    __pyx_v_error_status = vorbis_analysis(__pyx_v_vorbis_block, NULL);

    /* "pyrogg.pyx":253
 *         while error_status == 1:
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 */
    __pyx_t_1 = ((__pyx_v_error_status < 0) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":254
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
 *             if error_status < 0:
 *                 return error_status             # <<<<<<<<<<<<<<
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 *             if error_status < 0:
 */
      __pyx_r = __pyx_v_error_status;
      goto __pyx_L0;

      /* "pyrogg.pyx":253
 *         while error_status == 1:
 *             error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 */
    }

    /* "pyrogg.pyx":255
 *             if error_status < 0:
 *                 return error_status
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)             # <<<<<<<<<<<<<<
 *             if error_status < 0:
 *                 return error_status
 */
    __pyx_v_error_status = vorbis_bitrate_addblock(__pyx_v_vorbis_block);

    /* "pyrogg.pyx":256
 *                 return error_status
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 */
    __pyx_t_1 = ((__pyx_v_error_status < 0) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":257
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 *             if error_status < 0:
 *                 return error_status             # <<<<<<<<<<<<<<
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 *             if error_status < 0:
 */
      __pyx_r = __pyx_v_error_status;
      goto __pyx_L0;

      /* "pyrogg.pyx":256
 *                 return error_status
 *             error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 */
    }

    /* "pyrogg.pyx":258
 *             if error_status < 0:
 *                 return error_status
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)             # <<<<<<<<<<<<<<
 *             if error_status < 0:
 *                 return error_status
 */
    __pyx_v_error_status = __pyx_f_6pyrogg_14_VorbisRecoder__writeOggPackets(__pyx_v_self, __pyx_v_status, __pyx_v_stream_state, __pyx_v_dsp_state);

    /* "pyrogg.pyx":259
 *                 return error_status
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             eos = error_status
 */
    __pyx_t_1 = ((__pyx_v_error_status < 0) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":260
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 *             if error_status < 0:
 *                 return error_status             # <<<<<<<<<<<<<<
 *             eos = error_status
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 */
      __pyx_r = __pyx_v_error_status;
      goto __pyx_L0;

      /* "pyrogg.pyx":259
 *                 return error_status
 *             error_status = self._writeOggPackets(status, stream_state, dsp_state)
 *             if error_status < 0:             # <<<<<<<<<<<<<<
 *                 return error_status
 *             eos = error_status
 */
    }

    /* "pyrogg.pyx":261
 *             if error_status < 0:
 *                 return error_status
 *             eos = error_status             # <<<<<<<<<<<<<<
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         if error_status < 0:
 */
    __pyx_v_eos = __pyx_v_error_status;

    /* "pyrogg.pyx":262
 *                 return error_status
 *             eos = error_status
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)             # <<<<<<<<<<<<<<
 *         if error_status < 0:
 *             return error_status
 */
    __pyx_v_error_status = vorbis_analysis_blockout(__pyx_v_dsp_state, __pyx_v_vorbis_block);
  }

  /* "pyrogg.pyx":263
 *             eos = error_status
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         if error_status < 0:             # <<<<<<<<<<<<<<
 *             return error_status
 *         return eos
 */
  __pyx_t_1 = ((__pyx_v_error_status < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":264
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         if error_status < 0:
 *             return error_status             # <<<<<<<<<<<<<<
 *         return eos
 * 
 */
    __pyx_r = __pyx_v_error_status;
    goto __pyx_L0;

    /* "pyrogg.pyx":263
 *             eos = error_status
 *             error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
 *         if error_status < 0:             # <<<<<<<<<<<<<<
 *             return error_status
 *         return eos
 */
  }

  /* "pyrogg.pyx":265
 *         if error_status < 0:
 *             return error_status
 *         return eos             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
  __pyx_r = __pyx_v_eos;
  goto __pyx_L0;

  /* "pyrogg.pyx":244
 * 
 *     @cython.final
 *     cdef int _encodeVorbisBlocks(self, void* status,             # <<<<<<<<<<<<<<
 *                                  ogg.ogg_stream_state* stream_state,
 *                                  vorbis.vorbis_dsp_state* dsp_state,
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pyrogg.pyx":268
 * 
 *     @cython.final
 *     cdef int _writeOggPackets(self, void* status,             # <<<<<<<<<<<<<<
 *                               ogg.ogg_stream_state* stream_state,
 *                               vorbis.vorbis_dsp_state* dsp_state) nogil:
 */

static int __pyx_f_6pyrogg_14_VorbisRecoder__writeOggPackets(struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, void *__pyx_v_status, ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state) {
  ogg_packet __pyx_v_ogg_packet;
  ogg_page __pyx_v_ogg_page;
  int __pyx_v_eos;
  int __pyx_v_error_status;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "pyrogg.pyx":274
 *         cdef ogg.ogg_page   ogg_page
 *         cdef int eos, error_status
 *         eos = 0             # <<<<<<<<<<<<<<
 *         error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         while error_status > 0:
 */
  __pyx_v_eos = 0;

  /* "pyrogg.pyx":275
 *         cdef int eos, error_status
 *         eos = 0
 *         error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)             # <<<<<<<<<<<<<<
 *         while error_status > 0:
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:
 */
  __pyx_v_error_status = vorbis_bitrate_flushpacket(__pyx_v_dsp_state, (&__pyx_v_ogg_packet));

  /* "pyrogg.pyx":276
 *         eos = 0
 *         error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         while error_status > 0:             # <<<<<<<<<<<<<<
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:
 *                 return -1
 */
  while (1) {
    __pyx_t_1 = ((__pyx_v_error_status > 0) != 0);
    if (!__pyx_t_1) break;

    /* "pyrogg.pyx":277
 *         error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         while error_status > 0:
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:             # <<<<<<<<<<<<<<
 *                 return -1
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 */
    __pyx_t_1 = ((ogg_stream_packetin(__pyx_v_stream_state, (&__pyx_v_ogg_packet)) < 0) != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":278
 *         while error_status > 0:
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:
 *                 return -1             # <<<<<<<<<<<<<<
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 *                 if self._write(status, &ogg_page) < 0:
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "pyrogg.pyx":277
 *         error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         while error_status > 0:
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:             # <<<<<<<<<<<<<<
 *                 return -1
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 */
    }

    /* "pyrogg.pyx":279
 *             if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:
 *                 return -1
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:             # <<<<<<<<<<<<<<
 *                 if self._write(status, &ogg_page) < 0:
 *                     return -1
 */
    while (1) {
      __pyx_t_2 = ((ogg_stream_pageout(__pyx_v_stream_state, (&__pyx_v_ogg_page)) != 0) != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_1 = __pyx_t_2;
        goto __pyx_L8_bool_binop_done;
      }
      __pyx_t_2 = ((!(__pyx_v_eos != 0)) != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L8_bool_binop_done:;
      if (!__pyx_t_1) break;

      /* "pyrogg.pyx":280
 *                 return -1
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 *                 if self._write(status, &ogg_page) < 0:             # <<<<<<<<<<<<<<
 *                     return -1
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 */
      __pyx_t_1 = ((__pyx_v_self->_write(__pyx_v_status, (&__pyx_v_ogg_page)) < 0) != 0);
      if (__pyx_t_1) {

        /* "pyrogg.pyx":281
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 *                 if self._write(status, &ogg_page) < 0:
 *                     return -1             # <<<<<<<<<<<<<<
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 */
        __pyx_r = -1;
        goto __pyx_L0;

        /* "pyrogg.pyx":280
 *                 return -1
 *             while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
 *                 if self._write(status, &ogg_page) < 0:             # <<<<<<<<<<<<<<
 *                     return -1
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 */
      }

      /* "pyrogg.pyx":282
 *                 if self._write(status, &ogg_page) < 0:
 *                     return -1
 *                 eos = ogg.ogg_page_eos(&ogg_page)             # <<<<<<<<<<<<<<
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         if error_status < 0:
 */
      __pyx_v_eos = ogg_page_eos((&__pyx_v_ogg_page));
    }

    /* "pyrogg.pyx":283
 *                     return -1
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)             # <<<<<<<<<<<<<<
 *         if error_status < 0:
 *             return error_status
 */
    __pyx_v_error_status = vorbis_bitrate_flushpacket(__pyx_v_dsp_state, (&__pyx_v_ogg_packet));
  }

  /* "pyrogg.pyx":284
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         if error_status < 0:             # <<<<<<<<<<<<<<
 *             return error_status
 *         return eos
 */
  __pyx_t_1 = ((__pyx_v_error_status < 0) != 0);
  if (__pyx_t_1) {

    /* "pyrogg.pyx":285
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         if error_status < 0:
 *             return error_status             # <<<<<<<<<<<<<<
 *         return eos
 * 
 */
    __pyx_r = __pyx_v_error_status;
    goto __pyx_L0;

    /* "pyrogg.pyx":284
 *                 eos = ogg.ogg_page_eos(&ogg_page)
 *             error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
 *         if error_status < 0:             # <<<<<<<<<<<<<<
 *             return error_status
 *         return eos
 */
  }

  /* "pyrogg.pyx":286
 *         if error_status < 0:
 *             return error_status
 *         return eos             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_eos;
  goto __pyx_L0;

  /* "pyrogg.pyx":268
 * 
 *     @cython.final
 *     cdef int _writeOggPackets(self, void* status,             # <<<<<<<<<<<<<<
 *                               ogg.ogg_stream_state* stream_state,
 *                               vorbis.vorbis_dsp_state* dsp_state) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_14_VorbisRecoder_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6pyrogg_14_VorbisRecoder_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_14_VorbisRecoder___reduce_cython__(((struct __pyx_obj_6pyrogg__VorbisRecoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_14_VorbisRecoder___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg._VorbisRecoder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_14_VorbisRecoder_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6pyrogg_14_VorbisRecoder_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_14_VorbisRecoder_2__setstate_cython__(((struct __pyx_obj_6pyrogg__VorbisRecoder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_14_VorbisRecoder_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg__VorbisRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg._VorbisRecoder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":292
 *     cdef object _input_filename
 * 
 *     def __cinit__(self, input_filename):             # <<<<<<<<<<<<<<
 *         self._write = _writeToFile
 *         self._input_filename = _encodeFilename(input_filename)
 */

/* Python wrapper */
static int __pyx_pw_6pyrogg_17VorbisFileRecoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6pyrogg_17VorbisFileRecoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_input_filename = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_input_filename,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_input_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 292, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_input_filename = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 292, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_17VorbisFileRecoder___cinit__(((struct __pyx_obj_6pyrogg_VorbisFileRecoder *)__pyx_v_self), __pyx_v_input_filename);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6pyrogg_17VorbisFileRecoder___cinit__(struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, PyObject *__pyx_v_input_filename) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pyrogg.pyx":293
 * 
 *     def __cinit__(self, input_filename):
 *         self._write = _writeToFile             # <<<<<<<<<<<<<<
 *         self._input_filename = _encodeFilename(input_filename)
 * 
 */
  __pyx_v_self->__pyx_base._write = __pyx_f_6pyrogg__writeToFile;

  /* "pyrogg.pyx":294
 *     def __cinit__(self, input_filename):
 *         self._write = _writeToFile
 *         self._input_filename = _encodeFilename(input_filename)             # <<<<<<<<<<<<<<
 * 
 *     def recode(self, output_filename, int quality):
 */
  __pyx_t_1 = __pyx_f_6pyrogg__encodeFilename(__pyx_v_input_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_input_filename);
  __Pyx_DECREF(__pyx_v_self->_input_filename);
  __pyx_v_self->_input_filename = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":292
 *     cdef object _input_filename
 * 
 *     def __cinit__(self, input_filename):             # <<<<<<<<<<<<<<
 *         self._write = _writeToFile
 *         self._input_filename = _encodeFilename(input_filename)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":296
 *         self._input_filename = _encodeFilename(input_filename)
 * 
 *     def recode(self, output_filename, int quality):             # <<<<<<<<<<<<<<
 *         cdef vorbis.OggVorbis_File vorbisfile
 *         cdef stdio.FILE* cinfile
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_3recode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_3recode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_output_filename = 0;
  int __pyx_v_quality;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("recode (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_output_filename,&__pyx_n_s_quality,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_output_filename)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_quality)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("recode", 1, 2, 2, 1); __PYX_ERR(0, 296, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "recode") < 0)) __PYX_ERR(0, 296, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_output_filename = values[0];
    __pyx_v_quality = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_quality == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 296, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("recode", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 296, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.recode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_17VorbisFileRecoder_2recode(((struct __pyx_obj_6pyrogg_VorbisFileRecoder *)__pyx_v_self), __pyx_v_output_filename, __pyx_v_quality);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_2recode(struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, PyObject *__pyx_v_output_filename, int __pyx_v_quality) {
  OggVorbis_File __pyx_v_vorbisfile;
  FILE *__pyx_v_cinfile;
  FILE *__pyx_v_coutfile;
  int __pyx_v_result;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  __Pyx_RefNannySetupContext("recode", 0);
  __Pyx_INCREF(__pyx_v_output_filename);

  /* "pyrogg.pyx":301
 *         cdef stdio.FILE* coutfile
 * 
 *         output_filename = _encodeFilename(output_filename)             # <<<<<<<<<<<<<<
 *         cinfile = stdio.fopen(self._input_filename, "r")
 *         if cinfile is NULL:
 */
  __pyx_t_1 = __pyx_f_6pyrogg__encodeFilename(__pyx_v_output_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_output_filename, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":302
 * 
 *         output_filename = _encodeFilename(output_filename)
 *         cinfile = stdio.fopen(self._input_filename, "r")             # <<<<<<<<<<<<<<
 *         if cinfile is NULL:
 *             cpython.exc.PyErr_SetFromErrno(IOError)
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_self->_input_filename); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 302, __pyx_L1_error)
  __pyx_v_cinfile = fopen(__pyx_t_2, ((char const *)"r"));

  /* "pyrogg.pyx":303
 *         output_filename = _encodeFilename(output_filename)
 *         cinfile = stdio.fopen(self._input_filename, "r")
 *         if cinfile is NULL:             # <<<<<<<<<<<<<<
 *             cpython.exc.PyErr_SetFromErrno(IOError)
 * 
 */
  __pyx_t_3 = ((__pyx_v_cinfile == NULL) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":304
 *         cinfile = stdio.fopen(self._input_filename, "r")
 *         if cinfile is NULL:
 *             cpython.exc.PyErr_SetFromErrno(IOError)             # <<<<<<<<<<<<<<
 * 
 *         result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)
 */
    __pyx_t_4 = PyErr_SetFromErrno(__pyx_builtin_IOError); if (unlikely(__pyx_t_4 == ((PyObject *)NULL))) __PYX_ERR(0, 304, __pyx_L1_error)

    /* "pyrogg.pyx":303
 *         output_filename = _encodeFilename(output_filename)
 *         cinfile = stdio.fopen(self._input_filename, "r")
 *         if cinfile is NULL:             # <<<<<<<<<<<<<<
 *             cpython.exc.PyErr_SetFromErrno(IOError)
 * 
 */
  }

  /* "pyrogg.pyx":306
 *             cpython.exc.PyErr_SetFromErrno(IOError)
 * 
 *         result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)             # <<<<<<<<<<<<<<
 *         if result == vorbis.OV_ENOTVORBIS:
 *             stdio.fclose(cinfile)
 */
  __pyx_v_result = ov_open(__pyx_v_cinfile, (&__pyx_v_vorbisfile), NULL, 0);

  /* "pyrogg.pyx":307
 * 
 *         result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)
 *         if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *             stdio.fclose(cinfile)
 *             raise VorbisException(
 */
  __pyx_t_3 = ((__pyx_v_result == OV_ENOTVORBIS) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":308
 *         result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)
 *         if result == vorbis.OV_ENOTVORBIS:
 *             stdio.fclose(cinfile)             # <<<<<<<<<<<<<<
 *             raise VorbisException(
 *                 "%r is not a Vorbis file" % self._input_filename)
 */
    fclose(__pyx_v_cinfile);

    /* "pyrogg.pyx":309
 *         if result == vorbis.OV_ENOTVORBIS:
 *             stdio.fclose(cinfile)
 *             raise VorbisException(             # <<<<<<<<<<<<<<
 *                 "%r is not a Vorbis file" % self._input_filename)
 *         if result < 0:
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "pyrogg.pyx":310
 *             stdio.fclose(cinfile)
 *             raise VorbisException(
 *                 "%r is not a Vorbis file" % self._input_filename)             # <<<<<<<<<<<<<<
 *         if result < 0:
 *             stdio.fclose(cinfile)
 */
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_r_is_not_a_Vorbis_file, __pyx_v_self->_input_filename); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_7) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 309, __pyx_L1_error)

    /* "pyrogg.pyx":307
 * 
 *         result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)
 *         if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *             stdio.fclose(cinfile)
 *             raise VorbisException(
 */
  }

  /* "pyrogg.pyx":311
 *             raise VorbisException(
 *                 "%r is not a Vorbis file" % self._input_filename)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             stdio.fclose(cinfile)
 *             raise IOError("Error reading from file %r" % self._input_filename)
 */
  __pyx_t_3 = ((__pyx_v_result < 0) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":312
 *                 "%r is not a Vorbis file" % self._input_filename)
 *         if result < 0:
 *             stdio.fclose(cinfile)             # <<<<<<<<<<<<<<
 *             raise IOError("Error reading from file %r" % self._input_filename)
 * 
 */
    fclose(__pyx_v_cinfile);

    /* "pyrogg.pyx":313
 *         if result < 0:
 *             stdio.fclose(cinfile)
 *             raise IOError("Error reading from file %r" % self._input_filename)             # <<<<<<<<<<<<<<
 * 
 *         coutfile = stdio.fopen(output_filename, "w")
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Error_reading_from_file_r, __pyx_v_self->_input_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 313, __pyx_L1_error)

    /* "pyrogg.pyx":311
 *             raise VorbisException(
 *                 "%r is not a Vorbis file" % self._input_filename)
 *         if result < 0:             # <<<<<<<<<<<<<<
 *             stdio.fclose(cinfile)
 *             raise IOError("Error reading from file %r" % self._input_filename)
 */
  }

  /* "pyrogg.pyx":315
 *             raise IOError("Error reading from file %r" % self._input_filename)
 * 
 *         coutfile = stdio.fopen(output_filename, "w")             # <<<<<<<<<<<<<<
 *         if coutfile == NULL:
 *             vorbis.ov_clear(&vorbisfile) # closes the input file
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_output_filename); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 315, __pyx_L1_error)
  __pyx_v_coutfile = fopen(__pyx_t_2, ((char const *)"w"));

  /* "pyrogg.pyx":316
 * 
 *         coutfile = stdio.fopen(output_filename, "w")
 *         if coutfile == NULL:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&vorbisfile) # closes the input file
 *             raise IOError("Error opening output file %r" % output_filename)
 */
  __pyx_t_3 = ((__pyx_v_coutfile == NULL) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":317
 *         coutfile = stdio.fopen(output_filename, "w")
 *         if coutfile == NULL:
 *             vorbis.ov_clear(&vorbisfile) # closes the input file             # <<<<<<<<<<<<<<
 *             raise IOError("Error opening output file %r" % output_filename)
 * 
 */
    ov_clear((&__pyx_v_vorbisfile));

    /* "pyrogg.pyx":318
 *         if coutfile == NULL:
 *             vorbis.ov_clear(&vorbisfile) # closes the input file
 *             raise IOError("Error opening output file %r" % output_filename)             # <<<<<<<<<<<<<<
 * 
 *         t = 0
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Error_opening_output_file_r, __pyx_v_output_filename); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 318, __pyx_L1_error)

    /* "pyrogg.pyx":316
 * 
 *         coutfile = stdio.fopen(output_filename, "w")
 *         if coutfile == NULL:             # <<<<<<<<<<<<<<
 *             vorbis.ov_clear(&vorbisfile) # closes the input file
 *             raise IOError("Error opening output file %r" % output_filename)
 */
  }

  /* "pyrogg.pyx":320
 *             raise IOError("Error opening output file %r" % output_filename)
 * 
 *         t = 0             # <<<<<<<<<<<<<<
 *         try:
 *             t = self._recode(coutfile, &vorbisfile, quality)
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_t = __pyx_int_0;

  /* "pyrogg.pyx":321
 * 
 *         t = 0
 *         try:             # <<<<<<<<<<<<<<
 *             t = self._recode(coutfile, &vorbisfile, quality)
 *         finally:
 */
  /*try:*/ {

    /* "pyrogg.pyx":322
 *         t = 0
 *         try:
 *             t = self._recode(coutfile, &vorbisfile, quality)             # <<<<<<<<<<<<<<
 *         finally:
 *             stdio.fclose(coutfile)
 */
    __pyx_t_1 = __pyx_f_6pyrogg_14_VorbisRecoder__recode(((struct __pyx_obj_6pyrogg__VorbisRecoder *)__pyx_v_self), __pyx_v_coutfile, (&__pyx_v_vorbisfile), __pyx_v_quality); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L8_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_t, __pyx_t_1);
    __pyx_t_1 = 0;
  }

  /* "pyrogg.pyx":324
 *             t = self._recode(coutfile, &vorbisfile, quality)
 *         finally:
 *             stdio.fclose(coutfile)             # <<<<<<<<<<<<<<
 *         return t
 * 
 */
  /*finally:*/ {
    /*normal exit:*/{
      fclose(__pyx_v_coutfile);
      goto __pyx_L9;
    }
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __pyx_L8_error:;
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __Pyx_PyThreadState_assign
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
      {
        fclose(__pyx_v_coutfile);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      }
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
      __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
      goto __pyx_L1_error;
    }
    __pyx_L9:;
  }

  /* "pyrogg.pyx":325
 *         finally:
 *             stdio.fclose(coutfile)
 *         return t             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "pyrogg.pyx":296
 *         self._input_filename = _encodeFilename(input_filename)
 * 
 *     def recode(self, output_filename, int quality):             # <<<<<<<<<<<<<<
 *         cdef vorbis.OggVorbis_File vorbisfile
 *         cdef stdio.FILE* cinfile
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.recode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_output_filename);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_17VorbisFileRecoder_4__reduce_cython__(((struct __pyx_obj_6pyrogg_VorbisFileRecoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6pyrogg_17VorbisFileRecoder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_17VorbisFileRecoder_6__setstate_cython__(((struct __pyx_obj_6pyrogg_VorbisFileRecoder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_17VorbisFileRecoder_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFileRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFileRecoder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":328
 * 
 * 
 * cdef int _writeToFile(void* coutfile, ogg.ogg_page* ogg_page) nogil:             # <<<<<<<<<<<<<<
 *     cdef size_t count
 *     count = ogg_page.header_len
 */

static int __pyx_f_6pyrogg__writeToFile(void *__pyx_v_coutfile, ogg_page *__pyx_v_ogg_page) {
  size_t __pyx_v_count;
  int __pyx_r;
  long __pyx_t_1;
  int __pyx_t_2;

  /* "pyrogg.pyx":330
 * cdef int _writeToFile(void* coutfile, ogg.ogg_page* ogg_page) nogil:
 *     cdef size_t count
 *     count = ogg_page.header_len             # <<<<<<<<<<<<<<
 *     if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:
 *         return -1
 */
  __pyx_t_1 = __pyx_v_ogg_page->header_len;
  __pyx_v_count = __pyx_t_1;

  /* "pyrogg.pyx":331
 *     cdef size_t count
 *     count = ogg_page.header_len
 *     if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:             # <<<<<<<<<<<<<<
 *         return -1
 *     count = ogg_page.body_len
 */
  __pyx_t_2 = ((fwrite(__pyx_v_ogg_page->header, 1, __pyx_v_count, ((FILE *)__pyx_v_coutfile)) < __pyx_v_count) != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":332
 *     count = ogg_page.header_len
 *     if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:
 *         return -1             # <<<<<<<<<<<<<<
 *     count = ogg_page.body_len
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyrogg.pyx":331
 *     cdef size_t count
 *     count = ogg_page.header_len
 *     if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:             # <<<<<<<<<<<<<<
 *         return -1
 *     count = ogg_page.body_len
 */
  }

  /* "pyrogg.pyx":333
 *     if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:
 *         return -1
 *     count = ogg_page.body_len             # <<<<<<<<<<<<<<
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:
 *         return -1
 */
  __pyx_t_1 = __pyx_v_ogg_page->body_len;
  __pyx_v_count = __pyx_t_1;

  /* "pyrogg.pyx":334
 *         return -1
 *     count = ogg_page.body_len
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:             # <<<<<<<<<<<<<<
 *         return -1
 *     return 0
 */
  __pyx_t_2 = ((fwrite(__pyx_v_ogg_page->body, 1, __pyx_v_count, ((FILE *)__pyx_v_coutfile)) < __pyx_v_count) != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":335
 *     count = ogg_page.body_len
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:
 *         return -1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "pyrogg.pyx":334
 *         return -1
 *     count = ogg_page.body_len
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:             # <<<<<<<<<<<<<<
 *         return -1
 *     return 0
 */
  }

  /* "pyrogg.pyx":336
 *     if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:
 *         return -1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyrogg.pyx":328
 * 
 * 
 * cdef int _writeToFile(void* coutfile, ogg.ogg_page* ogg_page) nogil:             # <<<<<<<<<<<<<<
 *     cdef size_t count
 *     count = ogg_page.header_len
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pyrogg.pyx":347
 *     cdef object exception
 * 
 *     def __cinit__(self, f):             # <<<<<<<<<<<<<<
 *         self.read  = f.read
 *         self.tell  = f.tell
 */

/* Python wrapper */
static int __pyx_pw_6pyrogg_14FilelikeReader_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6pyrogg_14FilelikeReader_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_f = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_f,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 347, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_f = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 347, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.FilelikeReader.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_14FilelikeReader___cinit__(((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_v_self), __pyx_v_f);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6pyrogg_14FilelikeReader___cinit__(struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, PyObject *__pyx_v_f) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pyrogg.pyx":348
 * 
 *     def __cinit__(self, f):
 *         self.read  = f.read             # <<<<<<<<<<<<<<
 *         self.tell  = f.tell
 *         self.seek  = f.seek
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_read); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->read);
  __Pyx_DECREF(__pyx_v_self->read);
  __pyx_v_self->read = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":349
 *     def __cinit__(self, f):
 *         self.read  = f.read
 *         self.tell  = f.tell             # <<<<<<<<<<<<<<
 *         self.seek  = f.seek
 *         try:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_tell); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->tell);
  __Pyx_DECREF(__pyx_v_self->tell);
  __pyx_v_self->tell = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":350
 *         self.read  = f.read
 *         self.tell  = f.tell
 *         self.seek  = f.seek             # <<<<<<<<<<<<<<
 *         try:
 *             self.close = f.close
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_seek); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->seek);
  __Pyx_DECREF(__pyx_v_self->seek);
  __pyx_v_self->seek = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":351
 *         self.tell  = f.tell
 *         self.seek  = f.seek
 *         try:             # <<<<<<<<<<<<<<
 *             self.close = f.close
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyrogg.pyx":352
 *         self.seek  = f.seek
 *         try:
 *             self.close = f.close             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             pass
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_close); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->close);
      __Pyx_DECREF(__pyx_v_self->close);
      __pyx_v_self->close = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "pyrogg.pyx":351
 *         self.tell  = f.tell
 *         self.seek  = f.seek
 *         try:             # <<<<<<<<<<<<<<
 *             self.close = f.close
 *         except AttributeError:
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyrogg.pyx":353
 *         try:
 *             self.close = f.close
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             pass
 * 
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_5) {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":351
 *         self.tell  = f.tell
 *         self.seek  = f.seek
 *         try:             # <<<<<<<<<<<<<<
 *             self.close = f.close
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L8_try_end:;
  }

  /* "pyrogg.pyx":347
 *     cdef object exception
 * 
 *     def __cinit__(self, f):             # <<<<<<<<<<<<<<
 *         self.read  = f.read
 *         self.tell  = f.tell
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.FilelikeReader.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":356
 *             pass
 * 
 *     cdef void _storeException(self, exception):             # <<<<<<<<<<<<<<
 *         if self.exception is None:
 *             if exception is None:
 */

static void __pyx_f_6pyrogg_14FilelikeReader__storeException(struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, PyObject *__pyx_v_exception) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("_storeException", 0);

  /* "pyrogg.pyx":357
 * 
 *     cdef void _storeException(self, exception):
 *         if self.exception is None:             # <<<<<<<<<<<<<<
 *             if exception is None:
 *                 self.exception = sys.exc_info()
 */
  __pyx_t_1 = (__pyx_v_self->exception == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":358
 *     cdef void _storeException(self, exception):
 *         if self.exception is None:
 *             if exception is None:             # <<<<<<<<<<<<<<
 *                 self.exception = sys.exc_info()
 *             else:
 */
    __pyx_t_2 = (__pyx_v_exception == Py_None);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "pyrogg.pyx":359
 *         if self.exception is None:
 *             if exception is None:
 *                 self.exception = sys.exc_info()             # <<<<<<<<<<<<<<
 *             else:
 *                 self.exception = exception
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_6pyrogg_sys, __pyx_n_s_exc_info); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (__pyx_t_5) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else {
        __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GIVEREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_v_self->exception);
      __Pyx_DECREF(__pyx_v_self->exception);
      __pyx_v_self->exception = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "pyrogg.pyx":358
 *     cdef void _storeException(self, exception):
 *         if self.exception is None:
 *             if exception is None:             # <<<<<<<<<<<<<<
 *                 self.exception = sys.exc_info()
 *             else:
 */
      goto __pyx_L4;
    }

    /* "pyrogg.pyx":361
 *                 self.exception = sys.exc_info()
 *             else:
 *                 self.exception = exception             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_v_exception);
      __Pyx_GIVEREF(__pyx_v_exception);
      __Pyx_GOTREF(__pyx_v_self->exception);
      __Pyx_DECREF(__pyx_v_self->exception);
      __pyx_v_self->exception = __pyx_v_exception;
    }
    __pyx_L4:;

    /* "pyrogg.pyx":357
 * 
 *     cdef void _storeException(self, exception):
 *         if self.exception is None:             # <<<<<<<<<<<<<<
 *             if exception is None:
 *                 self.exception = sys.exc_info()
 */
  }

  /* "pyrogg.pyx":356
 *             pass
 * 
 *     cdef void _storeException(self, exception):             # <<<<<<<<<<<<<<
 *         if self.exception is None:
 *             if exception is None:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pyrogg.FilelikeReader._storeException", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_14FilelikeReader_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6pyrogg_14FilelikeReader_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_14FilelikeReader_2__reduce_cython__(((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_14FilelikeReader_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.FilelikeReader.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_14FilelikeReader_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6pyrogg_14FilelikeReader_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_14FilelikeReader_4__setstate_cython__(((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_14FilelikeReader_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.FilelikeReader.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":367
 *     cdef object f
 * 
 *     def __cinit__(self, f):             # <<<<<<<<<<<<<<
 *         if not hasattr(f, 'read') or \
 *                not hasattr(f, 'tell') or \
 */

/* Python wrapper */
static int __pyx_pw_6pyrogg_21VorbisFilelikeRecoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6pyrogg_21VorbisFilelikeRecoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_f = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_f,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 367, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_f = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 367, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_21VorbisFilelikeRecoder___cinit__(((struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)__pyx_v_self), __pyx_v_f);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6pyrogg_21VorbisFilelikeRecoder___cinit__(struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, PyObject *__pyx_v_f) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pyrogg.pyx":368
 * 
 *     def __cinit__(self, f):
 *         if not hasattr(f, 'read') or \             # <<<<<<<<<<<<<<
 *                not hasattr(f, 'tell') or \
 *                not hasattr(f, 'seek'):
 */
  __pyx_t_2 = __Pyx_HasAttr(__pyx_v_f, __pyx_n_s_read); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 368, __pyx_L1_error)
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }

  /* "pyrogg.pyx":369
 *     def __cinit__(self, f):
 *         if not hasattr(f, 'read') or \
 *                not hasattr(f, 'tell') or \             # <<<<<<<<<<<<<<
 *                not hasattr(f, 'seek'):
 *             raise TypeError, "Seekable file-like object required, got %s" % type(f)
 */
  __pyx_t_3 = __Pyx_HasAttr(__pyx_v_f, __pyx_n_s_tell); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_t_2 = ((!(__pyx_t_3 != 0)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "pyrogg.pyx":370
 *         if not hasattr(f, 'read') or \
 *                not hasattr(f, 'tell') or \
 *                not hasattr(f, 'seek'):             # <<<<<<<<<<<<<<
 *             raise TypeError, "Seekable file-like object required, got %s" % type(f)
 *         self.f = f
 */
  __pyx_t_2 = __Pyx_HasAttr(__pyx_v_f, __pyx_n_s_seek); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 370, __pyx_L1_error)
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;

  /* "pyrogg.pyx":368
 * 
 *     def __cinit__(self, f):
 *         if not hasattr(f, 'read') or \             # <<<<<<<<<<<<<<
 *                not hasattr(f, 'tell') or \
 *                not hasattr(f, 'seek'):
 */
  if (__pyx_t_1) {

    /* "pyrogg.pyx":371
 *                not hasattr(f, 'tell') or \
 *                not hasattr(f, 'seek'):
 *             raise TypeError, "Seekable file-like object required, got %s" % type(f)             # <<<<<<<<<<<<<<
 *         self.f = f
 *         self._write = _writeToFilelike
 */
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Seekable_file_like_object_requir, ((PyObject *)Py_TYPE(__pyx_v_f))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_builtin_TypeError, __pyx_t_4, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 371, __pyx_L1_error)

    /* "pyrogg.pyx":368
 * 
 *     def __cinit__(self, f):
 *         if not hasattr(f, 'read') or \             # <<<<<<<<<<<<<<
 *                not hasattr(f, 'tell') or \
 *                not hasattr(f, 'seek'):
 */
  }

  /* "pyrogg.pyx":372
 *                not hasattr(f, 'seek'):
 *             raise TypeError, "Seekable file-like object required, got %s" % type(f)
 *         self.f = f             # <<<<<<<<<<<<<<
 *         self._write = _writeToFilelike
 * 
 */
  __Pyx_INCREF(__pyx_v_f);
  __Pyx_GIVEREF(__pyx_v_f);
  __Pyx_GOTREF(__pyx_v_self->f);
  __Pyx_DECREF(__pyx_v_self->f);
  __pyx_v_self->f = __pyx_v_f;

  /* "pyrogg.pyx":373
 *             raise TypeError, "Seekable file-like object required, got %s" % type(f)
 *         self.f = f
 *         self._write = _writeToFilelike             # <<<<<<<<<<<<<<
 * 
 *     def recode(self, f, int quality):
 */
  __pyx_v_self->__pyx_base._write = __pyx_f_6pyrogg__writeToFilelike;

  /* "pyrogg.pyx":367
 *     cdef object f
 * 
 *     def __cinit__(self, f):             # <<<<<<<<<<<<<<
 *         if not hasattr(f, 'read') or \
 *                not hasattr(f, 'tell') or \
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":375
 *         self._write = _writeToFilelike
 * 
 *     def recode(self, f, int quality):             # <<<<<<<<<<<<<<
 *         cdef vorbis.OggVorbis_File vorbisfile
 *         cdef FilelikeReader reader
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_3recode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_3recode(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_f = 0;
  int __pyx_v_quality;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("recode (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_f,&__pyx_n_s_quality,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_quality)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("recode", 1, 2, 2, 1); __PYX_ERR(0, 375, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "recode") < 0)) __PYX_ERR(0, 375, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_f = values[0];
    __pyx_v_quality = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_quality == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("recode", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 375, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.recode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6pyrogg_21VorbisFilelikeRecoder_2recode(((struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)__pyx_v_self), __pyx_v_f, __pyx_v_quality);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_2recode(struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, PyObject *__pyx_v_f, int __pyx_v_quality) {
  OggVorbis_File __pyx_v_vorbisfile;
  struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_reader = 0;
  PyObject *__pyx_v_write = NULL;
  int __pyx_v_result;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("recode", 0);

  /* "pyrogg.pyx":378
 *         cdef vorbis.OggVorbis_File vorbisfile
 *         cdef FilelikeReader reader
 *         write = f.write # make sure it's there             # <<<<<<<<<<<<<<
 *         if not callable(write):
 *             raise TypeError("writable file-like object required, got %s" % type(f).__name__)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_write); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_write = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":379
 *         cdef FilelikeReader reader
 *         write = f.write # make sure it's there
 *         if not callable(write):             # <<<<<<<<<<<<<<
 *             raise TypeError("writable file-like object required, got %s" % type(f).__name__)
 *         reader = FilelikeReader(self.f)
 */
  __pyx_t_2 = __Pyx_PyCallable_Check(__pyx_v_write); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 379, __pyx_L1_error)
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":380
 *         write = f.write # make sure it's there
 *         if not callable(write):
 *             raise TypeError("writable file-like object required, got %s" % type(f).__name__)             # <<<<<<<<<<<<<<
 *         reader = FilelikeReader(self.f)
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_f)), __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_writable_file_like_object_requir, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 380, __pyx_L1_error)

    /* "pyrogg.pyx":379
 *         cdef FilelikeReader reader
 *         write = f.write # make sure it's there
 *         if not callable(write):             # <<<<<<<<<<<<<<
 *             raise TypeError("writable file-like object required, got %s" % type(f).__name__)
 *         reader = FilelikeReader(self.f)
 */
  }

  /* "pyrogg.pyx":381
 *         if not callable(write):
 *             raise TypeError("writable file-like object required, got %s" % type(f).__name__)
 *         reader = FilelikeReader(self.f)             # <<<<<<<<<<<<<<
 * 
 *         result = vorbis.ov_open_callbacks(<void*>reader, &vorbisfile,
 */
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->f);
  __Pyx_GIVEREF(__pyx_v_self->f);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->f);
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6pyrogg_FilelikeReader), __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_reader = ((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":383
 *         reader = FilelikeReader(self.f)
 * 
 *         result = vorbis.ov_open_callbacks(<void*>reader, &vorbisfile,             # <<<<<<<<<<<<<<
 *                                           NULL, 0, filelike_callbacks)
 *         if result == vorbis.OV_ENOTVORBIS:
 */
  __pyx_v_result = ov_open_callbacks(((void *)__pyx_v_reader), (&__pyx_v_vorbisfile), NULL, 0, __pyx_v_6pyrogg_filelike_callbacks);

  /* "pyrogg.pyx":385
 *         result = vorbis.ov_open_callbacks(<void*>reader, &vorbisfile,
 *                                           NULL, 0, filelike_callbacks)
 *         if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:
 */
  __pyx_t_3 = ((__pyx_v_result == OV_ENOTVORBIS) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":386
 *                                           NULL, 0, filelike_callbacks)
 *         if result == vorbis.OV_ENOTVORBIS:
 *             raise VorbisException("file is not a Vorbis file")             # <<<<<<<<<<<<<<
 *         elif result < 0:
 *             raise IOError("Error reading from file-like")
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 386, __pyx_L1_error)

    /* "pyrogg.pyx":385
 *         result = vorbis.ov_open_callbacks(<void*>reader, &vorbisfile,
 *                                           NULL, 0, filelike_callbacks)
 *         if result == vorbis.OV_ENOTVORBIS:             # <<<<<<<<<<<<<<
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:
 */
  }

  /* "pyrogg.pyx":387
 *         if result == vorbis.OV_ENOTVORBIS:
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:             # <<<<<<<<<<<<<<
 *             raise IOError("Error reading from file-like")
 * 
 */
  __pyx_t_3 = ((__pyx_v_result < 0) != 0);
  if (__pyx_t_3) {

    /* "pyrogg.pyx":388
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:
 *             raise IOError("Error reading from file-like")             # <<<<<<<<<<<<<<
 * 
 *         t = self._recode(<void*>write, &vorbisfile, quality)
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_IOError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 388, __pyx_L1_error)

    /* "pyrogg.pyx":387
 *         if result == vorbis.OV_ENOTVORBIS:
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:             # <<<<<<<<<<<<<<
 *             raise IOError("Error reading from file-like")
 * 
 */
  }

  /* "pyrogg.pyx":390
 *             raise IOError("Error reading from file-like")
 * 
 *         t = self._recode(<void*>write, &vorbisfile, quality)             # <<<<<<<<<<<<<<
 * 
 *         if reader.exception is not None:
 */
  __pyx_t_4 = __pyx_f_6pyrogg_14_VorbisRecoder__recode(((struct __pyx_obj_6pyrogg__VorbisRecoder *)__pyx_v_self), ((void *)__pyx_v_write), (&__pyx_v_vorbisfile), __pyx_v_quality); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_t = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pyrogg.pyx":392
 *         t = self._recode(<void*>write, &vorbisfile, quality)
 * 
 *         if reader.exception is not None:             # <<<<<<<<<<<<<<
 *             raise reader.exception[0], reader.exception[1], reader.exception[2]
 * 
 */
  __pyx_t_3 = (__pyx_v_reader->exception != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "pyrogg.pyx":393
 * 
 *         if reader.exception is not None:
 *             raise reader.exception[0], reader.exception[1], reader.exception[2]             # <<<<<<<<<<<<<<
 * 
 *         return t
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_reader->exception, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_reader->exception, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_reader->exception, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_4, __pyx_t_1, __pyx_t_5, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 393, __pyx_L1_error)

    /* "pyrogg.pyx":392
 *         t = self._recode(<void*>write, &vorbisfile, quality)
 * 
 *         if reader.exception is not None:             # <<<<<<<<<<<<<<
 *             raise reader.exception[0], reader.exception[1], reader.exception[2]
 * 
 */
  }

  /* "pyrogg.pyx":395
 *             raise reader.exception[0], reader.exception[1], reader.exception[2]
 * 
 *         return t             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "pyrogg.pyx":375
 *         self._write = _writeToFilelike
 * 
 *     def recode(self, f, int quality):             # <<<<<<<<<<<<<<
 *         cdef vorbis.OggVorbis_File vorbisfile
 *         cdef FilelikeReader reader
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.recode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_reader);
  __Pyx_XDECREF(__pyx_v_write);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_21VorbisFilelikeRecoder_4__reduce_cython__(((struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6pyrogg_21VorbisFilelikeRecoder_6__setstate_cython__(((struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6pyrogg_21VorbisFilelikeRecoder_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyrogg.VorbisFilelikeRecoder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyrogg.pyx":398
 * 
 * 
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:             # <<<<<<<<<<<<<<
 *     try:
 *         write = <object>outfile_write
 */

static int __pyx_f_6pyrogg__writeToFilelike(void *__pyx_v_outfile_write, ogg_page *__pyx_v_ogg_page) {
  PyObject *__pyx_v_write = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_writeToFilelike", 0);

  /* "pyrogg.pyx":399
 * 
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:
 *     try:             # <<<<<<<<<<<<<<
 *         write = <object>outfile_write
 *         write(ogg_page.header[:ogg_page.header_len])
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyrogg.pyx":400
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:
 *     try:
 *         write = <object>outfile_write             # <<<<<<<<<<<<<<
 *         write(ogg_page.header[:ogg_page.header_len])
 *         write(ogg_page.body[:ogg_page.body_len])
 */
      __pyx_t_4 = ((PyObject *)__pyx_v_outfile_write);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_write = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "pyrogg.pyx":401
 *     try:
 *         write = <object>outfile_write
 *         write(ogg_page.header[:ogg_page.header_len])             # <<<<<<<<<<<<<<
 *         write(ogg_page.body[:ogg_page.body_len])
 *         return 0
 */
      __pyx_t_5 = __Pyx_PyBytes_FromStringAndSize(((const char*)__pyx_v_ogg_page->header) + 0, __pyx_v_ogg_page->header_len - 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 401, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_write);
      __pyx_t_6 = __pyx_v_write; __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_5};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_5};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 401, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 401, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyrogg.pyx":402
 *         write = <object>outfile_write
 *         write(ogg_page.header[:ogg_page.header_len])
 *         write(ogg_page.body[:ogg_page.body_len])             # <<<<<<<<<<<<<<
 *         return 0
 *     except:
 */
      __pyx_t_6 = __Pyx_PyBytes_FromStringAndSize(((const char*)__pyx_v_ogg_page->body) + 0, __pyx_v_ogg_page->body_len - 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 402, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_write);
      __pyx_t_8 = __pyx_v_write; __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 402, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyrogg.pyx":403
 *         write(ogg_page.header[:ogg_page.header_len])
 *         write(ogg_page.body[:ogg_page.body_len])
 *         return 0             # <<<<<<<<<<<<<<
 *     except:
 *         return -1
 */
      __pyx_r = 0;
      goto __pyx_L7_try_return;

      /* "pyrogg.pyx":399
 * 
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:
 *     try:             # <<<<<<<<<<<<<<
 *         write = <object>outfile_write
 *         write(ogg_page.header[:ogg_page.header_len])
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyrogg.pyx":404
 *         write(ogg_page.body[:ogg_page.body_len])
 *         return 0
 *     except:             # <<<<<<<<<<<<<<
 *         return -1
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("pyrogg._writeToFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_8, &__pyx_t_7) < 0) __PYX_ERR(0, 404, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_7);

      /* "pyrogg.pyx":405
 *         return 0
 *     except:
 *         return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_r = -1;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L6_except_return;
    }
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":399
 * 
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:
 *     try:             # <<<<<<<<<<<<<<
 *         write = <object>outfile_write
 *         write(ogg_page.header[:ogg_page.header_len])
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "pyrogg.pyx":398
 * 
 * 
 * cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:             # <<<<<<<<<<<<<<
 *     try:
 *         write = <object>outfile_write
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("pyrogg._writeToFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_write);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "pyrogg.pyx":408
 * 
 * 
 * cdef size_t _readFilelike(void* dest, size_t size, size_t count,             # <<<<<<<<<<<<<<
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 */

static size_t __pyx_f_6pyrogg__readFilelike(void *__pyx_v_dest, size_t __pyx_v_size, size_t __pyx_v_count, void *__pyx_v_creader) {
  struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_reader = 0;
  PyObject *__pyx_v_data = NULL;
  Py_ssize_t __pyx_v_length;
  CYTHON_UNUSED long __pyx_v_errno;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  char *__pyx_t_12;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_readFilelike", 0);

  /* "pyrogg.pyx":410
 * cdef size_t _readFilelike(void* dest, size_t size, size_t count,
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader             # <<<<<<<<<<<<<<
 *     try:
 *         data = reader.read(size * count)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_creader);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_reader = ((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":411
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         data = reader.read(size * count)
 *         if data is None:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyrogg.pyx":412
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 *         data = reader.read(size * count)             # <<<<<<<<<<<<<<
 *         if data is None:
 *             return 0
 */
      __pyx_t_5 = __Pyx_PyInt_FromSize_t((__pyx_v_size * __pyx_v_count)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 412, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_v_reader->read);
      __pyx_t_6 = __pyx_v_reader->read; __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 412, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_v_data = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "pyrogg.pyx":413
 *     try:
 *         data = reader.read(size * count)
 *         if data is None:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif isinstance(data, bytes):
 */
      __pyx_t_9 = (__pyx_v_data == Py_None);
      __pyx_t_10 = (__pyx_t_9 != 0);
      if (__pyx_t_10) {

        /* "pyrogg.pyx":414
 *         data = reader.read(size * count)
 *         if data is None:
 *             return 0             # <<<<<<<<<<<<<<
 *         elif isinstance(data, bytes):
 *             length = len(<bytes>data)
 */
        __pyx_r = 0;
        goto __pyx_L7_try_return;

        /* "pyrogg.pyx":413
 *     try:
 *         data = reader.read(size * count)
 *         if data is None:             # <<<<<<<<<<<<<<
 *             return 0
 *         elif isinstance(data, bytes):
 */
      }

      /* "pyrogg.pyx":415
 *         if data is None:
 *             return 0
 *         elif isinstance(data, bytes):             # <<<<<<<<<<<<<<
 *             length = len(<bytes>data)
 *             if length > size * count:
 */
      __pyx_t_10 = PyBytes_Check(__pyx_v_data); 
      __pyx_t_9 = (__pyx_t_10 != 0);
      if (__pyx_t_9) {

        /* "pyrogg.pyx":416
 *             return 0
 *         elif isinstance(data, bytes):
 *             length = len(<bytes>data)             # <<<<<<<<<<<<<<
 *             if length > size * count:
 *                 reader._storeException(
 */
        if (unlikely(__pyx_v_data == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 416, __pyx_L3_error)
        }
        __pyx_t_11 = PyBytes_GET_SIZE(((PyObject*)__pyx_v_data)); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 416, __pyx_L3_error)
        __pyx_v_length = __pyx_t_11;

        /* "pyrogg.pyx":417
 *         elif isinstance(data, bytes):
 *             length = len(<bytes>data)
 *             if length > size * count:             # <<<<<<<<<<<<<<
 *                 reader._storeException(
 *                     ValueError("Returned string is too long"))
 */
        __pyx_t_9 = ((__pyx_v_length > (__pyx_v_size * __pyx_v_count)) != 0);
        if (__pyx_t_9) {

          /* "pyrogg.pyx":419
 *             if length > size * count:
 *                 reader._storeException(
 *                     ValueError("Returned string is too long"))             # <<<<<<<<<<<<<<
 *             else:
 *                 if length > 0:
 */
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);

          /* "pyrogg.pyx":418
 *             length = len(<bytes>data)
 *             if length > size * count:
 *                 reader._storeException(             # <<<<<<<<<<<<<<
 *                     ValueError("Returned string is too long"))
 *             else:
 */
          __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, __pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pyrogg.pyx":417
 *         elif isinstance(data, bytes):
 *             length = len(<bytes>data)
 *             if length > size * count:             # <<<<<<<<<<<<<<
 *                 reader._storeException(
 *                     ValueError("Returned string is too long"))
 */
          goto __pyx_L10;
        }

        /* "pyrogg.pyx":421
 *                     ValueError("Returned string is too long"))
 *             else:
 *                 if length > 0:             # <<<<<<<<<<<<<<
 *                     memcpy(dest, <char*><bytes>data, length)
 *                 return length
 */
        /*else*/ {
          __pyx_t_9 = ((__pyx_v_length > 0) != 0);
          if (__pyx_t_9) {

            /* "pyrogg.pyx":422
 *             else:
 *                 if length > 0:
 *                     memcpy(dest, <char*><bytes>data, length)             # <<<<<<<<<<<<<<
 *                 return length
 *         else:
 */
            if (unlikely(__pyx_v_data == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
              __PYX_ERR(0, 422, __pyx_L3_error)
            }
            __pyx_t_12 = __Pyx_PyBytes_AsWritableString(__pyx_v_data); if (unlikely((!__pyx_t_12) && PyErr_Occurred())) __PYX_ERR(0, 422, __pyx_L3_error)
            memcpy(__pyx_v_dest, ((char *)__pyx_t_12), __pyx_v_length);

            /* "pyrogg.pyx":421
 *                     ValueError("Returned string is too long"))
 *             else:
 *                 if length > 0:             # <<<<<<<<<<<<<<
 *                     memcpy(dest, <char*><bytes>data, length)
 *                 return length
 */
          }

          /* "pyrogg.pyx":423
 *                 if length > 0:
 *                     memcpy(dest, <char*><bytes>data, length)
 *                 return length             # <<<<<<<<<<<<<<
 *         else:
 *             reader._storeException(
 */
          __pyx_r = __pyx_v_length;
          goto __pyx_L7_try_return;
        }
        __pyx_L10:;

        /* "pyrogg.pyx":415
 *         if data is None:
 *             return 0
 *         elif isinstance(data, bytes):             # <<<<<<<<<<<<<<
 *             length = len(<bytes>data)
 *             if length > size * count:
 */
        goto __pyx_L9;
      }

      /* "pyrogg.pyx":425
 *                 return length
 *         else:
 *             reader._storeException(             # <<<<<<<<<<<<<<
 *                 TypeError(
 *                     "Invalid return type from read(), bytes required, got %s"
 */
      /*else*/ {

        /* "pyrogg.pyx":428
 *                 TypeError(
 *                     "Invalid return type from read(), bytes required, got %s"
 *                     % type(data.__name__)))             # <<<<<<<<<<<<<<
 *     except:
 *         reader._storeException(None)
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_return_type_from_read_by, ((PyObject *)Py_TYPE(__pyx_t_1))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 428, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyrogg.pyx":426
 *         else:
 *             reader._storeException(
 *                 TypeError(             # <<<<<<<<<<<<<<
 *                     "Invalid return type from read(), bytes required, got %s"
 *                     % type(data.__name__)))
 */
        __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 426, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyrogg.pyx":425
 *                 return length
 *         else:
 *             reader._storeException(             # <<<<<<<<<<<<<<
 *                 TypeError(
 *                     "Invalid return type from read(), bytes required, got %s"
 */
        __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, __pyx_t_6);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __pyx_L9:;

      /* "pyrogg.pyx":411
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         data = reader.read(size * count)
 *         if data is None:
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyrogg.pyx":429
 *                     "Invalid return type from read(), bytes required, got %s"
 *                     % type(data.__name__)))
 *     except:             # <<<<<<<<<<<<<<
 *         reader._storeException(None)
 *     errno = -1  # propagate read error
 */
    /*except:*/ {
      __Pyx_AddTraceback("pyrogg._readFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_1, &__pyx_t_8) < 0) __PYX_ERR(0, 429, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);

      /* "pyrogg.pyx":430
 *                     % type(data.__name__)))
 *     except:
 *         reader._storeException(None)             # <<<<<<<<<<<<<<
 *     errno = -1  # propagate read error
 *     return 0
 */
      __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, Py_None);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L4_exception_handled;
    }
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":411
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         data = reader.read(size * count)
 *         if data is None:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L8_try_end:;
  }

  /* "pyrogg.pyx":431
 *     except:
 *         reader._storeException(None)
 *     errno = -1  # propagate read error             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_v_errno = -1L;

  /* "pyrogg.pyx":432
 *         reader._storeException(None)
 *     errno = -1  # propagate read error
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyrogg.pyx":408
 * 
 * 
 * cdef size_t _readFilelike(void* dest, size_t size, size_t count,             # <<<<<<<<<<<<<<
 *                           void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("pyrogg._readFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_reader);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "pyrogg.pyx":435
 * 
 * 
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

static int __pyx_f_6pyrogg__seekFilelike(void *__pyx_v_creader, ogg_int64_t __pyx_v_offset, int __pyx_v_whence) {
  struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_reader = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_seekFilelike", 0);

  /* "pyrogg.pyx":436
 * 
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader             # <<<<<<<<<<<<<<
 *     try:
 *         reader.seek(offset, whence)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_creader);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_reader = ((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":437
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(offset, whence)
 *         return 0
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyrogg.pyx":438
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 *         reader.seek(offset, whence)             # <<<<<<<<<<<<<<
 *         return 0
 *     except:
 */
      __pyx_t_5 = __Pyx_PyInt_From_ogg_int64_t(__pyx_v_offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 438, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_whence); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 438, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_reader->seek);
      __pyx_t_7 = __pyx_v_reader->seek; __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 438, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_6);
        __pyx_t_5 = 0;
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyrogg.pyx":439
 *     try:
 *         reader.seek(offset, whence)
 *         return 0             # <<<<<<<<<<<<<<
 *     except:
 *         reader._storeException(None)
 */
      __pyx_r = 0;
      goto __pyx_L7_try_return;

      /* "pyrogg.pyx":437
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(offset, whence)
 *         return 0
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyrogg.pyx":440
 *         reader.seek(offset, whence)
 *         return 0
 *     except:             # <<<<<<<<<<<<<<
 *         reader._storeException(None)
 *     return -1
 */
    /*except:*/ {
      __Pyx_AddTraceback("pyrogg._seekFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_7, &__pyx_t_10) < 0) __PYX_ERR(0, 440, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_10);

      /* "pyrogg.pyx":441
 *         return 0
 *     except:
 *         reader._storeException(None)             # <<<<<<<<<<<<<<
 *     return -1
 * 
 */
      __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L4_exception_handled;
    }
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":437
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(offset, whence)
 *         return 0
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
  }

  /* "pyrogg.pyx":442
 *     except:
 *         reader._storeException(None)
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "pyrogg.pyx":435
 * 
 * 
 * cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_WriteUnraisable("pyrogg._seekFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_reader);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "pyrogg.pyx":445
 * 
 * 
 * cdef long _tellFilelike(void* creader) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

static long __pyx_f_6pyrogg__tellFilelike(void *__pyx_v_creader) {
  struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_reader = 0;
  long __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  long __pyx_t_7;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_tellFilelike", 0);

  /* "pyrogg.pyx":446
 * 
 * cdef long _tellFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader             # <<<<<<<<<<<<<<
 *     try:
 *         return reader.tell()
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_creader);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_reader = ((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":447
 * cdef long _tellFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         return reader.tell()
 *     except:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyrogg.pyx":448
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 *         return reader.tell()             # <<<<<<<<<<<<<<
 *     except:
 *         reader._storeException(None)
 */
      __Pyx_INCREF(__pyx_v_reader->tell);
      __pyx_t_5 = __pyx_v_reader->tell; __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L3_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_7 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_7 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 448, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_r = __pyx_t_7;
      goto __pyx_L7_try_return;

      /* "pyrogg.pyx":447
 * cdef long _tellFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         return reader.tell()
 *     except:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyrogg.pyx":449
 *     try:
 *         return reader.tell()
 *     except:             # <<<<<<<<<<<<<<
 *         reader._storeException(None)
 *     return -1
 */
    /*except:*/ {
      __Pyx_AddTraceback("pyrogg._tellFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_5, &__pyx_t_6) < 0) __PYX_ERR(0, 449, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);

      /* "pyrogg.pyx":450
 *         return reader.tell()
 *     except:
 *         reader._storeException(None)             # <<<<<<<<<<<<<<
 *     return -1
 * 
 */
      __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L4_exception_handled;
    }
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":447
 * cdef long _tellFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         return reader.tell()
 *     except:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
  }

  /* "pyrogg.pyx":451
 *     except:
 *         reader._storeException(None)
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1L;
  goto __pyx_L0;

  /* "pyrogg.pyx":445
 * 
 * 
 * cdef long _tellFilelike(void* creader) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pyrogg._tellFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_reader);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "pyrogg.pyx":454
 * 
 * 
 * cdef int _closeFilelike(void* creader) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

static int __pyx_f_6pyrogg__closeFilelike(void *__pyx_v_creader) {
  struct __pyx_obj_6pyrogg_FilelikeReader *__pyx_v_reader = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_closeFilelike", 0);

  /* "pyrogg.pyx":455
 * 
 * cdef int _closeFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader             # <<<<<<<<<<<<<<
 *     try:
 *         reader.seek(0)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_creader);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_reader = ((struct __pyx_obj_6pyrogg_FilelikeReader *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyrogg.pyx":456
 * cdef int _closeFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(0)
 *         return 0
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyrogg.pyx":457
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 *         reader.seek(0)             # <<<<<<<<<<<<<<
 *         return 0
 *     except:
 */
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_v_reader->seek, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 457, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyrogg.pyx":458
 *     try:
 *         reader.seek(0)
 *         return 0             # <<<<<<<<<<<<<<
 *     except:
 *         reader._storeException(None)
 */
      __pyx_r = 0;
      goto __pyx_L7_try_return;

      /* "pyrogg.pyx":456
 * cdef int _closeFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(0)
 *         return 0
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyrogg.pyx":459
 *         reader.seek(0)
 *         return 0
 *     except:             # <<<<<<<<<<<<<<
 *         reader._storeException(None)
 *     return -1
 */
    /*except:*/ {
      __Pyx_AddTraceback("pyrogg._closeFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_5, &__pyx_t_6) < 0) __PYX_ERR(0, 459, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);

      /* "pyrogg.pyx":460
 *         return 0
 *     except:
 *         reader._storeException(None)             # <<<<<<<<<<<<<<
 *     return -1
 * 
 */
      __pyx_f_6pyrogg_14FilelikeReader__storeException(__pyx_v_reader, Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L4_exception_handled;
    }
    __pyx_L5_except_error:;

    /* "pyrogg.pyx":456
 * cdef int _closeFilelike(void* creader) with gil:
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:             # <<<<<<<<<<<<<<
 *         reader.seek(0)
 *         return 0
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
  }

  /* "pyrogg.pyx":461
 *     except:
 *         reader._storeException(None)
 *     return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "pyrogg.pyx":454
 * 
 * 
 * cdef int _closeFilelike(void* creader) with gil:             # <<<<<<<<<<<<<<
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("pyrogg._closeFilelike", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_reader);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "pyrogg.pyx":464
 * 
 * 
 * cdef void _splitStereo(char* decbuffer, float* channel1, float* channel2,             # <<<<<<<<<<<<<<
 *                        int samples) nogil:
 *     cdef int i
 */

static void __pyx_f_6pyrogg__splitStereo(char *__pyx_v_decbuffer, float *__pyx_v_channel1, float *__pyx_v_channel2, int __pyx_v_samples) {
  int __pyx_v_i;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "pyrogg.pyx":467
 *                        int samples) nogil:
 *     cdef int i
 *     for i in range(samples):             # <<<<<<<<<<<<<<
 *         channel1[i] = (
 *             ((decbuffer[i*4+1] << 8) | (0x00ff & <int>decbuffer[i*4]))
 */
  __pyx_t_1 = __pyx_v_samples;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "pyrogg.pyx":468
 *     cdef int i
 *     for i in range(samples):
 *         channel1[i] = (             # <<<<<<<<<<<<<<
 *             ((decbuffer[i*4+1] << 8) | (0x00ff & <int>decbuffer[i*4]))
 *             ) / 32768.0
 */
    (__pyx_v_channel1[__pyx_v_i]) = ((((__pyx_v_decbuffer[((__pyx_v_i * 4) + 1)]) << 8) | (0x00ff & ((int)(__pyx_v_decbuffer[(__pyx_v_i * 4)])))) / 32768.0);

    /* "pyrogg.pyx":472
 *             ) / 32768.0
 * 
 *         channel2[i] = (             # <<<<<<<<<<<<<<
 *             ((decbuffer[i*4+3] << 8) | (0x00ff & <int>decbuffer[i*4+2]))
 *             ) / 32768.0
 */
    (__pyx_v_channel2[__pyx_v_i]) = ((((__pyx_v_decbuffer[((__pyx_v_i * 4) + 3)]) << 8) | (0x00ff & ((int)(__pyx_v_decbuffer[((__pyx_v_i * 4) + 2)])))) / 32768.0);
  }

  /* "pyrogg.pyx":464
 * 
 * 
 * cdef void _splitStereo(char* decbuffer, float* channel1, float* channel2,             # <<<<<<<<<<<<<<
 *                        int samples) nogil:
 *     cdef int i
 */

  /* function exit code */
}

/* "pyrogg.pyx":477
 * 
 * 
 * cdef int _initVorbisEncoder(ogg.ogg_stream_state* stream_state,             # <<<<<<<<<<<<<<
 *                             vorbis.vorbis_dsp_state* dsp_state,
 *                             vorbis.vorbis_info* vorbis_info,
 */

static int __pyx_f_6pyrogg__initVorbisEncoder(ogg_stream_state *__pyx_v_stream_state, vorbis_dsp_state *__pyx_v_dsp_state, vorbis_info *__pyx_v_vorbis_info, int __pyx_v_serialno) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_initVorbisEncoder", 0);

  /* "pyrogg.pyx":481
 *                             vorbis.vorbis_info* vorbis_info,
 *                             int serialno):
 *     vorbis.vorbis_analysis_init(dsp_state, vorbis_info)             # <<<<<<<<<<<<<<
 *     return ogg.ogg_stream_init(stream_state, serialno)
 */
  vorbis_analysis_init(__pyx_v_dsp_state, __pyx_v_vorbis_info);

  /* "pyrogg.pyx":482
 *                             int serialno):
 *     vorbis.vorbis_analysis_init(dsp_state, vorbis_info)
 *     return ogg.ogg_stream_init(stream_state, serialno)             # <<<<<<<<<<<<<<
 */
  __pyx_r = ogg_stream_init(__pyx_v_stream_state, __pyx_v_serialno);
  goto __pyx_L0;

  /* "pyrogg.pyx":477
 * 
 * 
 * cdef int _initVorbisEncoder(ogg.ogg_stream_state* stream_state,             # <<<<<<<<<<<<<<
 *                             vorbis.vorbis_dsp_state* dsp_state,
 *                             vorbis.vorbis_info* vorbis_info,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_6pyrogg_VorbisFile(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6pyrogg_VorbisFile *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6pyrogg_VorbisFile *)o);
  p->filename = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6pyrogg_10VorbisFile_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6pyrogg_VorbisFile(PyObject *o) {
  struct __pyx_obj_6pyrogg_VorbisFile *p = (struct __pyx_obj_6pyrogg_VorbisFile *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_6pyrogg_10VorbisFile_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->filename);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6pyrogg_VorbisFile(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6pyrogg_VorbisFile *p = (struct __pyx_obj_6pyrogg_VorbisFile *)o;
  if (p->filename) {
    e = (*v)(p->filename, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6pyrogg_VorbisFile(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6pyrogg_VorbisFile *p = (struct __pyx_obj_6pyrogg_VorbisFile *)o;
  tmp = ((PyObject*)p->filename);
  p->filename = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6pyrogg_VorbisFile[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6pyrogg_10VorbisFile_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6pyrogg_10VorbisFile_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6pyrogg_VorbisFile = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyrogg.VorbisFile", /*tp_name*/
  sizeof(struct __pyx_obj_6pyrogg_VorbisFile), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6pyrogg_VorbisFile, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6pyrogg_VorbisFile, /*tp_traverse*/
  __pyx_tp_clear_6pyrogg_VorbisFile, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6pyrogg_VorbisFile, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6pyrogg_VorbisFile, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6pyrogg__VorbisRecoder __pyx_vtable_6pyrogg__VorbisRecoder;

static PyObject *__pyx_tp_new_6pyrogg__VorbisRecoder(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6pyrogg__VorbisRecoder *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6pyrogg__VorbisRecoder *)o);
  p->__pyx_vtab = __pyx_vtabptr_6pyrogg__VorbisRecoder;
  return o;
}

static void __pyx_tp_dealloc_6pyrogg__VorbisRecoder(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_6pyrogg__VorbisRecoder[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6pyrogg_14_VorbisRecoder_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6pyrogg_14_VorbisRecoder_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6pyrogg__VorbisRecoder = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyrogg._VorbisRecoder", /*tp_name*/
  sizeof(struct __pyx_obj_6pyrogg__VorbisRecoder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6pyrogg__VorbisRecoder, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6pyrogg__VorbisRecoder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6pyrogg__VorbisRecoder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6pyrogg_VorbisFileRecoder __pyx_vtable_6pyrogg_VorbisFileRecoder;

static PyObject *__pyx_tp_new_6pyrogg_VorbisFileRecoder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6pyrogg_VorbisFileRecoder *p;
  PyObject *o = __pyx_tp_new_6pyrogg__VorbisRecoder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6pyrogg_VorbisFileRecoder *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6pyrogg__VorbisRecoder*)__pyx_vtabptr_6pyrogg_VorbisFileRecoder;
  p->_input_filename = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6pyrogg_17VorbisFileRecoder_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6pyrogg_VorbisFileRecoder(PyObject *o) {
  struct __pyx_obj_6pyrogg_VorbisFileRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFileRecoder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_input_filename);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6pyrogg__VorbisRecoder(o);
}

static int __pyx_tp_traverse_6pyrogg_VorbisFileRecoder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6pyrogg_VorbisFileRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFileRecoder *)o;
  e = ((likely(__pyx_ptype_6pyrogg__VorbisRecoder)) ? ((__pyx_ptype_6pyrogg__VorbisRecoder->tp_traverse) ? __pyx_ptype_6pyrogg__VorbisRecoder->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6pyrogg_VorbisFileRecoder)); if (e) return e;
  if (p->_input_filename) {
    e = (*v)(p->_input_filename, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6pyrogg_VorbisFileRecoder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6pyrogg_VorbisFileRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFileRecoder *)o;
  if (likely(__pyx_ptype_6pyrogg__VorbisRecoder)) { if (__pyx_ptype_6pyrogg__VorbisRecoder->tp_clear) __pyx_ptype_6pyrogg__VorbisRecoder->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6pyrogg_VorbisFileRecoder);
  tmp = ((PyObject*)p->_input_filename);
  p->_input_filename = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6pyrogg_VorbisFileRecoder[] = {
  {"recode", (PyCFunction)__pyx_pw_6pyrogg_17VorbisFileRecoder_3recode, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6pyrogg_17VorbisFileRecoder_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6pyrogg_17VorbisFileRecoder_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6pyrogg_VorbisFileRecoder = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyrogg.VorbisFileRecoder", /*tp_name*/
  sizeof(struct __pyx_obj_6pyrogg_VorbisFileRecoder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6pyrogg_VorbisFileRecoder, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6pyrogg_VorbisFileRecoder, /*tp_traverse*/
  __pyx_tp_clear_6pyrogg_VorbisFileRecoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6pyrogg_VorbisFileRecoder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6pyrogg_VorbisFileRecoder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6pyrogg_FilelikeReader __pyx_vtable_6pyrogg_FilelikeReader;

static PyObject *__pyx_tp_new_6pyrogg_FilelikeReader(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6pyrogg_FilelikeReader *p;
  PyObject *o;
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6pyrogg_FilelikeReader *)o);
  p->__pyx_vtab = __pyx_vtabptr_6pyrogg_FilelikeReader;
  p->read = Py_None; Py_INCREF(Py_None);
  p->tell = Py_None; Py_INCREF(Py_None);
  p->seek = Py_None; Py_INCREF(Py_None);
  p->close = Py_None; Py_INCREF(Py_None);
  p->exception = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6pyrogg_14FilelikeReader_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6pyrogg_FilelikeReader(PyObject *o) {
  struct __pyx_obj_6pyrogg_FilelikeReader *p = (struct __pyx_obj_6pyrogg_FilelikeReader *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->read);
  Py_CLEAR(p->tell);
  Py_CLEAR(p->seek);
  Py_CLEAR(p->close);
  Py_CLEAR(p->exception);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6pyrogg_FilelikeReader(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6pyrogg_FilelikeReader *p = (struct __pyx_obj_6pyrogg_FilelikeReader *)o;
  if (p->read) {
    e = (*v)(p->read, a); if (e) return e;
  }
  if (p->tell) {
    e = (*v)(p->tell, a); if (e) return e;
  }
  if (p->seek) {
    e = (*v)(p->seek, a); if (e) return e;
  }
  if (p->close) {
    e = (*v)(p->close, a); if (e) return e;
  }
  if (p->exception) {
    e = (*v)(p->exception, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6pyrogg_FilelikeReader(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6pyrogg_FilelikeReader *p = (struct __pyx_obj_6pyrogg_FilelikeReader *)o;
  tmp = ((PyObject*)p->read);
  p->read = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->tell);
  p->tell = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->seek);
  p->seek = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->close);
  p->close = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->exception);
  p->exception = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6pyrogg_FilelikeReader[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6pyrogg_14FilelikeReader_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6pyrogg_14FilelikeReader_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6pyrogg_FilelikeReader = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyrogg.FilelikeReader", /*tp_name*/
  sizeof(struct __pyx_obj_6pyrogg_FilelikeReader), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6pyrogg_FilelikeReader, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6pyrogg_FilelikeReader, /*tp_traverse*/
  __pyx_tp_clear_6pyrogg_FilelikeReader, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6pyrogg_FilelikeReader, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6pyrogg_FilelikeReader, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6pyrogg_VorbisFilelikeRecoder __pyx_vtable_6pyrogg_VorbisFilelikeRecoder;

static PyObject *__pyx_tp_new_6pyrogg_VorbisFilelikeRecoder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *p;
  PyObject *o = __pyx_tp_new_6pyrogg__VorbisRecoder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6pyrogg__VorbisRecoder*)__pyx_vtabptr_6pyrogg_VorbisFilelikeRecoder;
  p->f = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_6pyrogg_VorbisFilelikeRecoder(PyObject *o) {
  struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->f);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6pyrogg__VorbisRecoder(o);
}

static int __pyx_tp_traverse_6pyrogg_VorbisFilelikeRecoder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)o;
  e = ((likely(__pyx_ptype_6pyrogg__VorbisRecoder)) ? ((__pyx_ptype_6pyrogg__VorbisRecoder->tp_traverse) ? __pyx_ptype_6pyrogg__VorbisRecoder->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6pyrogg_VorbisFilelikeRecoder)); if (e) return e;
  if (p->f) {
    e = (*v)(p->f, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6pyrogg_VorbisFilelikeRecoder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *p = (struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder *)o;
  if (likely(__pyx_ptype_6pyrogg__VorbisRecoder)) { if (__pyx_ptype_6pyrogg__VorbisRecoder->tp_clear) __pyx_ptype_6pyrogg__VorbisRecoder->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6pyrogg_VorbisFilelikeRecoder);
  tmp = ((PyObject*)p->f);
  p->f = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6pyrogg_VorbisFilelikeRecoder[] = {
  {"recode", (PyCFunction)__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_3recode, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6pyrogg_21VorbisFilelikeRecoder_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6pyrogg_VorbisFilelikeRecoder = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyrogg.VorbisFilelikeRecoder", /*tp_name*/
  sizeof(struct __pyx_obj_6pyrogg_VorbisFilelikeRecoder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6pyrogg_VorbisFilelikeRecoder, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6pyrogg_VorbisFilelikeRecoder, /*tp_traverse*/
  __pyx_tp_clear_6pyrogg_VorbisFilelikeRecoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6pyrogg_VorbisFilelikeRecoder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6pyrogg_VorbisFilelikeRecoder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_pyrogg(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_pyrogg},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "pyrogg",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_0_2, __pyx_k_0_2, sizeof(__pyx_k_0_2), 0, 0, 1, 0},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Error_opening_output_file_r, __pyx_k_Error_opening_output_file_r, sizeof(__pyx_k_Error_opening_output_file_r), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_reading_from_file_like, __pyx_k_Error_reading_from_file_like, sizeof(__pyx_k_Error_reading_from_file_like), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_reading_from_file_r, __pyx_k_Error_reading_from_file_r, sizeof(__pyx_k_Error_reading_from_file_r), 0, 0, 1, 0},
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_kp_s_Invalid_return_type_from_read_by, __pyx_k_Invalid_return_type_from_read_by, sizeof(__pyx_k_Invalid_return_type_from_read_by), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_Opening_file_r_failed, __pyx_k_Opening_file_r_failed, sizeof(__pyx_k_Opening_file_r_failed), 0, 0, 1, 0},
  {&__pyx_kp_s_Returned_string_is_too_long, __pyx_k_Returned_string_is_too_long, sizeof(__pyx_k_Returned_string_is_too_long), 0, 0, 1, 0},
  {&__pyx_kp_s_Seekable_file_like_object_requir, __pyx_k_Seekable_file_like_object_requir, sizeof(__pyx_k_Seekable_file_like_object_requir), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unknown_error_d_occurred_while_p, __pyx_k_Unknown_error_d_occurred_while_p, sizeof(__pyx_k_Unknown_error_d_occurred_while_p), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_VorbisDecodeException, __pyx_k_VorbisDecodeException, sizeof(__pyx_k_VorbisDecodeException), 0, 0, 1, 1},
  {&__pyx_n_s_VorbisException, __pyx_k_VorbisException, sizeof(__pyx_k_VorbisException), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_exc_info, __pyx_k_exc_info, sizeof(__pyx_k_exc_info), 0, 0, 1, 1},
  {&__pyx_kp_s_expected_file_path_got_s, __pyx_k_expected_file_path_got_s, sizeof(__pyx_k_expected_file_path_got_s), 0, 0, 1, 0},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_kp_s_file_is_not_a_Vorbis_file, __pyx_k_file_is_not_a_Vorbis_file, sizeof(__pyx_k_file_is_not_a_Vorbis_file), 0, 0, 1, 0},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_n_s_getfilesystemencoding, __pyx_k_getfilesystemencoding, sizeof(__pyx_k_getfilesystemencoding), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_input_filename, __pyx_k_input_filename, sizeof(__pyx_k_input_filename), 0, 0, 1, 1},
  {&__pyx_kp_s_iso8859_1, __pyx_k_iso8859_1, sizeof(__pyx_k_iso8859_1), 0, 0, 1, 0},
  {&__pyx_kp_s_lost_sync, __pyx_k_lost_sync, sizeof(__pyx_k_lost_sync), 0, 0, 1, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_output_filename, __pyx_k_output_filename, sizeof(__pyx_k_output_filename), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pyrogg, __pyx_k_pyrogg, sizeof(__pyx_k_pyrogg), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_quality, __pyx_k_quality, sizeof(__pyx_k_quality), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_kp_s_r_is_not_a_Vorbis_file, __pyx_k_r_is_not_a_Vorbis_file, sizeof(__pyx_k_r_is_not_a_Vorbis_file), 0, 0, 1, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_read, __pyx_k_read, sizeof(__pyx_k_read), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_seek, __pyx_k_seek, sizeof(__pyx_k_seek), 0, 0, 1, 1},
  {&__pyx_kp_s_self__write_cannot_be_converted, __pyx_k_self__write_cannot_be_converted, sizeof(__pyx_k_self__write_cannot_be_converted), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_tell, __pyx_k_tell, sizeof(__pyx_k_tell), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 0, 1, 1},
  {&__pyx_kp_s_writable_file_like_object_requir, __pyx_k_writable_file_like_object_requir, sizeof(__pyx_k_writable_file_like_object_requir), 0, 0, 1, 0},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 52, __pyx_L1_error)
  __pyx_builtin_IOError = __Pyx_GetBuiltinName(__pyx_n_s_IOError); if (!__pyx_builtin_IOError) __PYX_ERR(0, 65, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 131, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 353, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 467, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "pyrogg.pyx":211
 *         read_status = com.read_status[com.buffer_ready]
 *         if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:
 *             raise VorbisDecodeException("lost sync")             # <<<<<<<<<<<<<<
 * 
 *     @cython.final
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_lost_sync); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self__write_cannot_be_converted); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self._write cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self__write_cannot_be_converted); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "pyrogg.pyx":386
 *                                           NULL, 0, filelike_callbacks)
 *         if result == vorbis.OV_ENOTVORBIS:
 *             raise VorbisException("file is not a Vorbis file")             # <<<<<<<<<<<<<<
 *         elif result < 0:
 *             raise IOError("Error reading from file-like")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_file_is_not_a_Vorbis_file); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "pyrogg.pyx":388
 *             raise VorbisException("file is not a Vorbis file")
 *         elif result < 0:
 *             raise IOError("Error reading from file-like")             # <<<<<<<<<<<<<<
 * 
 *         t = self._recode(<void*>write, &vorbisfile, quality)
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_Error_reading_from_file_like); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pyrogg.pyx":419
 *             if length > size * count:
 *                 reader._storeException(
 *                     ValueError("Returned string is too long"))             # <<<<<<<<<<<<<<
 *             else:
 *                 if length > 0:
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Returned_string_is_too_long); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "pyrogg.pyx":457
 *     cdef FilelikeReader reader = <FilelikeReader>creader
 *     try:
 *         reader.seek(0)             # <<<<<<<<<<<<<<
 *         return 0
 *     except:
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_int_0); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  /* InitThreads.init */
  #ifdef WITH_THREAD
PyEval_InitThreads();
#endif

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initpyrogg(void); /*proto*/
PyMODINIT_FUNC initpyrogg(void)
#else
PyMODINIT_FUNC PyInit_pyrogg(void); /*proto*/
PyMODINIT_FUNC PyInit_pyrogg(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_pyrogg(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_pyrogg(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("pyrogg", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyrogg) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyrogg")) {
      if (unlikely(PyDict_SetItemString(modules, "pyrogg", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  __pyx_v_6pyrogg_time = Py_None; Py_INCREF(Py_None);
  __pyx_v_6pyrogg_sys = Py_None; Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_6pyrogg_VorbisFile) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  __pyx_type_6pyrogg_VorbisFile.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "VorbisFile", (PyObject *)&__pyx_type_6pyrogg_VorbisFile) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6pyrogg_VorbisFile) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  __pyx_ptype_6pyrogg_VorbisFile = &__pyx_type_6pyrogg_VorbisFile;
  __pyx_vtabptr_6pyrogg__VorbisRecoder = &__pyx_vtable_6pyrogg__VorbisRecoder;
  __pyx_vtable_6pyrogg__VorbisRecoder._recode = (PyObject *(*)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, OggVorbis_File *, int))__pyx_f_6pyrogg_14_VorbisRecoder__recode;
  __pyx_vtable_6pyrogg__VorbisRecoder._recodeParallel = (PyObject *(*)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, char *, OggVorbis_File *, vorbis_dsp_state, vorbis_block, ogg_stream_state))__pyx_f_6pyrogg_14_VorbisRecoder__recodeParallel;
  __pyx_vtable_6pyrogg__VorbisRecoder._writeVorbisHeader = (int (*)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_comment *))__pyx_f_6pyrogg_14_VorbisRecoder__writeVorbisHeader;
  __pyx_vtable_6pyrogg__VorbisRecoder._encodeVorbisBlocks = (int (*)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *, vorbis_block *))__pyx_f_6pyrogg_14_VorbisRecoder__encodeVorbisBlocks;
  __pyx_vtable_6pyrogg__VorbisRecoder._writeOggPackets = (int (*)(struct __pyx_obj_6pyrogg__VorbisRecoder *, void *, ogg_stream_state *, vorbis_dsp_state *))__pyx_f_6pyrogg_14_VorbisRecoder__writeOggPackets;
  if (PyType_Ready(&__pyx_type_6pyrogg__VorbisRecoder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __pyx_type_6pyrogg__VorbisRecoder.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_6pyrogg__VorbisRecoder.tp_dict, __pyx_vtabptr_6pyrogg__VorbisRecoder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_VorbisRecoder", (PyObject *)&__pyx_type_6pyrogg__VorbisRecoder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6pyrogg__VorbisRecoder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __pyx_ptype_6pyrogg__VorbisRecoder = &__pyx_type_6pyrogg__VorbisRecoder;
  __pyx_vtabptr_6pyrogg_VorbisFileRecoder = &__pyx_vtable_6pyrogg_VorbisFileRecoder;
  __pyx_vtable_6pyrogg_VorbisFileRecoder.__pyx_base = *__pyx_vtabptr_6pyrogg__VorbisRecoder;
  __pyx_type_6pyrogg_VorbisFileRecoder.tp_base = __pyx_ptype_6pyrogg__VorbisRecoder;
  if (PyType_Ready(&__pyx_type_6pyrogg_VorbisFileRecoder) < 0) __PYX_ERR(0, 289, __pyx_L1_error)
  __pyx_type_6pyrogg_VorbisFileRecoder.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_6pyrogg_VorbisFileRecoder.tp_dict, __pyx_vtabptr_6pyrogg_VorbisFileRecoder) < 0) __PYX_ERR(0, 289, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "VorbisFileRecoder", (PyObject *)&__pyx_type_6pyrogg_VorbisFileRecoder) < 0) __PYX_ERR(0, 289, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6pyrogg_VorbisFileRecoder) < 0) __PYX_ERR(0, 289, __pyx_L1_error)
  __pyx_ptype_6pyrogg_VorbisFileRecoder = &__pyx_type_6pyrogg_VorbisFileRecoder;
  __pyx_vtabptr_6pyrogg_FilelikeReader = &__pyx_vtable_6pyrogg_FilelikeReader;
  __pyx_vtable_6pyrogg_FilelikeReader._storeException = (void (*)(struct __pyx_obj_6pyrogg_FilelikeReader *, PyObject *))__pyx_f_6pyrogg_14FilelikeReader__storeException;
  if (PyType_Ready(&__pyx_type_6pyrogg_FilelikeReader) < 0) __PYX_ERR(0, 340, __pyx_L1_error)
  __pyx_type_6pyrogg_FilelikeReader.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_6pyrogg_FilelikeReader.tp_dict, __pyx_vtabptr_6pyrogg_FilelikeReader) < 0) __PYX_ERR(0, 340, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FilelikeReader", (PyObject *)&__pyx_type_6pyrogg_FilelikeReader) < 0) __PYX_ERR(0, 340, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6pyrogg_FilelikeReader) < 0) __PYX_ERR(0, 340, __pyx_L1_error)
  __pyx_ptype_6pyrogg_FilelikeReader = &__pyx_type_6pyrogg_FilelikeReader;
  __pyx_vtabptr_6pyrogg_VorbisFilelikeRecoder = &__pyx_vtable_6pyrogg_VorbisFilelikeRecoder;
  __pyx_vtable_6pyrogg_VorbisFilelikeRecoder.__pyx_base = *__pyx_vtabptr_6pyrogg__VorbisRecoder;
  __pyx_type_6pyrogg_VorbisFilelikeRecoder.tp_base = __pyx_ptype_6pyrogg__VorbisRecoder;
  if (PyType_Ready(&__pyx_type_6pyrogg_VorbisFilelikeRecoder) < 0) __PYX_ERR(0, 364, __pyx_L1_error)
  __pyx_type_6pyrogg_VorbisFilelikeRecoder.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_6pyrogg_VorbisFilelikeRecoder.tp_dict, __pyx_vtabptr_6pyrogg_VorbisFilelikeRecoder) < 0) __PYX_ERR(0, 364, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "VorbisFilelikeRecoder", (PyObject *)&__pyx_type_6pyrogg_VorbisFilelikeRecoder) < 0) __PYX_ERR(0, 364, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6pyrogg_VorbisFilelikeRecoder) < 0) __PYX_ERR(0, 364, __pyx_L1_error)
  __pyx_ptype_6pyrogg_VorbisFilelikeRecoder = &__pyx_type_6pyrogg_VorbisFilelikeRecoder;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(2, 9, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pyrogg.pyx":14
 * 
 * cdef object time
 * from time import time             # <<<<<<<<<<<<<<
 * 
 * cdef object sys
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_time);
  __Pyx_GIVEREF(__pyx_n_s_time);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_time);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_time, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_6pyrogg_time);
  __Pyx_DECREF_SET(__pyx_v_6pyrogg_time, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyrogg.pyx":17
 * 
 * cdef object sys
 * import sys             # <<<<<<<<<<<<<<
 * 
 * __version__ = "0.2"
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_6pyrogg_sys);
  __Pyx_DECREF_SET(__pyx_v_6pyrogg_sys, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyrogg.pyx":19
 * import sys
 * 
 * __version__ = "0.2"             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_version, __pyx_kp_s_0_2) < 0) __PYX_ERR(0, 19, __pyx_L1_error)

  /* "pyrogg.pyx":22
 * 
 * 
 * class VorbisException(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_VorbisException, __pyx_n_s_VorbisException, (PyObject *) NULL, __pyx_n_s_pyrogg, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_VorbisException, __pyx_t_2, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VorbisException, __pyx_t_4) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyrogg.pyx":25
 *     pass
 * 
 * class VorbisDecodeException(VorbisException):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_VorbisException); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_VorbisDecodeException, __pyx_n_s_VorbisDecodeException, (PyObject *) NULL, __pyx_n_s_pyrogg, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_VorbisDecodeException, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VorbisDecodeException, __pyx_t_4) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyrogg.pyx":32
 * 
 * cdef vorbis.ov_callbacks filelike_callbacks
 * filelike_callbacks.read_func  = _readFilelike             # <<<<<<<<<<<<<<
 * filelike_callbacks.seek_func  = _seekFilelike
 * filelike_callbacks.tell_func  = _tellFilelike
 */
  __pyx_v_6pyrogg_filelike_callbacks.read_func = __pyx_f_6pyrogg__readFilelike;

  /* "pyrogg.pyx":33
 * cdef vorbis.ov_callbacks filelike_callbacks
 * filelike_callbacks.read_func  = _readFilelike
 * filelike_callbacks.seek_func  = _seekFilelike             # <<<<<<<<<<<<<<
 * filelike_callbacks.tell_func  = _tellFilelike
 * filelike_callbacks.close_func = _closeFilelike
 */
  __pyx_v_6pyrogg_filelike_callbacks.seek_func = __pyx_f_6pyrogg__seekFilelike;

  /* "pyrogg.pyx":34
 * filelike_callbacks.read_func  = _readFilelike
 * filelike_callbacks.seek_func  = _seekFilelike
 * filelike_callbacks.tell_func  = _tellFilelike             # <<<<<<<<<<<<<<
 * filelike_callbacks.close_func = _closeFilelike
 * 
 */
  __pyx_v_6pyrogg_filelike_callbacks.tell_func = __pyx_f_6pyrogg__tellFilelike;

  /* "pyrogg.pyx":35
 * filelike_callbacks.seek_func  = _seekFilelike
 * filelike_callbacks.tell_func  = _tellFilelike
 * filelike_callbacks.close_func = _closeFilelike             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_6pyrogg_filelike_callbacks.close_func = __pyx_f_6pyrogg__closeFilelike;

  /* "pyrogg.pyx":1
 * cimport ogg, vorbis             # <<<<<<<<<<<<<<
 * 
 * cimport cpython.mem
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyrogg", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyrogg");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseDoubleKeywords */
    static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
    static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
    static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* GetException */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* SaveResetException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* None */
      static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* PyErrExceptionMatches */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* WriteUnraisableException */
      static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* GetAttr */
      static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
      static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* GetItemInt */
      static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* CallNextTpTraverse */
      static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
      static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
      static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* Import */
      static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
      static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
      static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
      static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
      static int __Pyx_CLineForTraceback(int c_line) {
#ifdef CYTHON_CLINE_IN_TRACEBACK
    return ((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0;
#else
    PyObject *use_cline;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *ptype, *pvalue, *ptraceback;
      PyObject *use_cline_obj;
      PyErr_Fetch(&ptype, &pvalue, &ptraceback);
      use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        use_cline = NULL;
      }
      PyErr_Restore(ptype, pvalue, ptraceback);
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    return c_line;
#endif
}

/* CodeObjectCache */
      static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
      #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        __Pyx_PyThreadState_Current, /*PyThreadState *tstate,*/
        py_code,                     /*PyCodeObject *code,*/
        __pyx_d,              /*PyObject *globals,*/
        0                            /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
      #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ogg_int64_t(ogg_int64_t value) {
    const ogg_int64_t neg_one = (ogg_int64_t) -1, const_zero = (ogg_int64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ogg_int64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ogg_int64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ogg_int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(ogg_int64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(ogg_int64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ogg_int64_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
      static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
      #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
      static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
      #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
      #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
      static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
